@@ -1,7 +1,7 @@
 /*
  * AUTHOR: Jay Schmidgall <jay.schmidgall@spdbump.sungardss.com>
  *
- * $Id: choice.c,v 1.16 2004/12/07 23:55:53 tobiasoed Exp $
+ * $Id: choice.c,v 1.33 2005/08/02 15:00:53 tobiasoed Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -11,6 +11,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+
 #ifdef USE_EDITRES
 #include <X11/Intrinsic.h>
 #include <X11/Xmu/Editres.h>
@@ -21,35 +22,73 @@
 #include <Xm/Label.h>
 #include <Xm/RowColumn.h>
 #include <Xm/Scale.h>
+#include <Xm/PushB.h>
 #include <Xm/ToggleB.h>
 #include <Xm/TextF.h>
 #include <Xm/Text.h>
-#include <Xbae/Matrix.h>
 
+#include <Xbae/Matrix.h>
 
 #ifdef WITH_DMALLOC
 #include <dmalloc.h>
 #endif
 
+#define USE_RENDER_TABLE 1
+#define VERBOSE 0
+
+#if (XmVERSION < 2)
+#define XmUNSPECIFIED_PIXEL     ((Pixel) (~0))
+#endif
+
 static String fallback[] = {
-	"Choice*mw.rows:		10",
-	"Choice*mw.columns:		10",
-	"Choice*mw.visibleRows:		5",
-	"Choice*mw.visibleColumns:	5",
-	"Choice*mw.columnWidths:	10, 5, 10, 5, 10, 5, "
-	"				10, 5, 10, 5, 10, 5",
-	"Choice*mw.columnLabels:	Zero, One, Two, Three, Four,"
-	"				Five, Six, Seven, Eight, Nine",
-	"Choice*mw.rowLabels:		0, 1, 2, 3, 4, 5, 6, 7, 8, 9",
-/*	"Choice*mw.fontList:		7x14",	*/
-	"Choice*mw.traverseFixedCells:	True",
-	"Choice*mw.topRow:	3",
-        "Choice*mw.evenRowBackground: Red",
-        "Choice*mw.oddRowBackground: Blue",
-        "Choice*mw.altRowCount: 2",
-        "Choice*mw.calcCursorPosition: True",
-        "Choice*mw.buttonLabels: True",
-        NULL
+    "Choice*mw.rows:          10",
+    "Choice*mw.columns:       10",
+    "Choice*mw.visibleRows:    5",
+    "Choice*mw.visibleColumns: 5",
+    "Choice*mw.columnWidths:  14, 14, 14, 7, 7, 7, 7, 7, 7, 7",
+    "Choice*mw.rowHeights:     2,  1,  1, 1, 1, 1, 1, 1, 1, 1",
+    "Choice*mw.rowLabels:    000,  1,  2, 3, 4, 5, 6, 7, 8, 9",
+    "Choice*mw.columnLabels:  Zero, One, Two,   Three, Four,"
+    "                         Five, Six, Seven, Eight, Nine",
+    "Choice*mw.columnAlignments:"
+        "    alignment_beginning, alignment_center, alignment_end,"
+        "    alignment_beginning, alignment_beginning, alignment_beginning,"
+        "    alignment_beginning, alignment_beginning, alignment_beginning,"
+        "    alignment_beginning",
+/*    "Choice*mw.fontList:        7x14",    */
+    "Choice*mw.traverseFixedCells:    True",
+    "Choice*mw.topRow:    0",
+    "Choice*mw.evenRowBackground: Red",
+    "Choice*mw.oddRowBackground: Blue",
+    "Choice*mw.altRowCount: 2",
+    "Choice*mw.calcCursorPosition: True",
+    "Choice*mw.buttonLabels: True",
+    "Choice*mw.rowHeightInPixels: False",
+    "Choice*mw.multiLineCell: True",
+    "Choice*mw.wrapType: wrap_none",
+    "Choice*mw.showArrows: True",
+    #if 1
+    #if USE_RENDER_TABLE
+    "Choice*mw.renderTable: labels",
+    "Choice*mw.renderTable.fontType:         FONT_IS_FONT",
+    "Choice*mw.renderTable.fontName:        -*-helvetica-medium-r-*-*-10-*-*-*-*-*-*-*",
+    "Choice*mw.renderTable.labels.fontType:  FONT_IS_FONT",
+    "Choice*mw.renderTable.labels.fontName: -*-helvetica-medium-r-*-*-14-*-*-*-*-*-*-*",
+    #else
+    "Choice*mw.labelFont:		-*-helvetica-bold-r-*-*-10-*-*-*-*-*-*-*",
+	"Choice*mw.fontList:		-*-helvetica-bold-r-*-*-14-*-*-*-*-*-*-*",
+    #endif
+    #else
+    "Choice*form.textRenderTable: labels",
+    "Choice*form.textRenderTable.fontType:         FONT_IS_FONT",
+    "Choice*form.textRenderTable.fontName:        -*-helvetica-medium-o-*-*-10-*-*-*-*-*-*-*",
+    "Choice*form.textRenderTable.labels.fontType:  FONT_IS_FONT",
+    "Choice*form.textRenderTable.labels.fontName: -*-helvetica-medium-o-*-*-14-*-*-*-*-*-*-*",
+    #endif
+    #if 0
+    "Choice*mw.sensitive: False",
+    #endif
+    NULL
 };
 
 void
@@ -57,26 +96,26 @@ LoadMatrix(Widget w)
 {
     String *cells[10];
     static String rows[10][10] = {
-	{ "a 0,Zero", "b 0,One",  "c 0,Two",    "d 0,Three",  "e 0,Four", 
+    { "a 0,Zero\nThis column is left aligned", "b 0,One\nThis column is center aligned",  "c 0,Two\nThis column is right aligned",    "d 0,Three",  "e 0,Four", 
     "f 0,Five", "g 0,Six",  "h 0, Seven", "i 0, Eight", "j 0, Nine" },
-	{ "b 1,Zero", "c 1,One",  "d 1,Two",    "e 1,Three",  "f 1,Four", 
+    { "b 1,Zero", "c 1,One",  "d 1,Two",    "e 1,Three",  "f 1,Four", 
     "g 1,Five", "h 1,Six",  "i 1, Seven", "j 1, Eight", "a 1, Nine" },
-	{ "c 2,Zero", "d 2,One",  "e 2,Two",    "f 2,Three",  "g 2,Four", 
+    { "c 2,Zero", "d 2,One",  "e 2,Two",    "f 2,Three",  "g 2,Four", 
     "h 2,Five", "i 2,Six",  "j 2, Seven", "a 2, Eight", "b 2, Nine" },
-	{ "d 3,Zero", "e 3,One",  "f 3,Two",    "g 3,Three",  "h 3,Four", 
+    { "d 3,Zero", "e 3,One",  "f 3,Two",    "g 3,Three",  "h 3,Four", 
     "i 3,Five", "j 3,Six",  "a 3, Seven", "b 3, Eight", "c 3, Nine" },
-	{ "e 4,Zero", "f 4,One",  "g 4,Two",    "h 4,Three",  "i 4,Four", 
+    { "e 4,Zero", "f 4,One",  "g 4,Two",    "h 4,Three",  "i 4,Four", 
     "j 4,Five", "a 4,Six",  "b 4, Seven", "c 4, Eight", "d 4, Nine" },
-	{ "f 5,Zero", "g 5,One",  "h 5,Two",    "i 5,Three",  "j 5,Four", 
+    { "f 5,Zero", "g 5,One",  "h 5,Two",    "i 5,Three",  "j 5,Four", 
     "a 5,Five", "b 5,Six",  "c 5, Seven", "d 5, Eight", "e 5, Nine" },
-	{ "g 6,Zero", "h 6,One",  "i 6,Two",    "j 6,Three",  "a 6,Four", 
+    { "g 6,Zero", "h 6,One",  "i 6,Two",    "j 6,Three",  "a 6,Four", 
     "b 6,Five", "c 6,Six",  "d 6, Seven", "e 6, Eight", "f 6, Nine" },
-	{ "h 7,Zero", "i 7,One",  "j 7,Two",    "a 7,Three",  "b 7,Four", 
+    { "h 7,Zero", "i 7,One",  "j 7,Two",    "a 7,Three",  "b 7,Four", 
     "c 7,Five", "d 7,Six",  "e 7, Seven", "f 7, Eight", "g 7, Nine" },
-	{ "i 8,Zero", "j 8,One",  "a 8,Two",    "b 8,Three",  "c 8,Four", 
+    { "i 8,Zero", "j 8,One",  "a 8,Two",    "b 8,Three",  "c 8,Four", 
     "d 8,Five", "e 8,Six",  "f 8, Seven", "g 8, Eight", "h 8, Nine" },
-	{ "j 9,Zero", "a 9,One",  "b 9,Two",    "c 9,Three",  "d 9,Four", 
-    "e 9,Five", "f 9,Six",  "g 9, Seven", "h 9, Eight", "i 9, Nine" }
+    { "j 9,Zero", "a 9,One",  "b 9,Two",    "c 9,Three",  "d 9,Four", 
+    "e 9,Five", "f 9,Six",  "g 9, Seven", "h 9, Eight", "i 9, Nine\nhaha" }
     };
 
     cells[0] = &rows[0][0];
@@ -91,7 +130,7 @@ LoadMatrix(Widget w)
     cells[9] = &rows[9][0];
     
     XtVaSetValues(w,
-		  XmNcells,     cells,
+    	  XmNcells,     cells,
 		  NULL);
 }
 
@@ -123,236 +162,157 @@ enum { SelectSelect, SelectHighlight, SelectArm };
 /* ================================================================================================
  * The following functions are for 'select mode' buttons
  */
+
 void
 UnarmAll(Widget w)
 {
     int rows, columns, r, c;
-    unsigned char *rowShadows, *columnShadows, **cellShadows;
-    unsigned char cellShadowType;
-    Pixel **backgrounds;
-
     XtVaGetValues(w,
-		  XmNrows, &rows,
-		  XmNcolumns, &columns,
-		  XmNcellShadowType, &cellShadowType,
-		  XmNcellShadowTypes, &cellShadows,
-		  XmNrowShadowTypes, &rowShadows,
-		  XmNcolumnShadowTypes, &columnShadows,
-		  XmNcellBackgrounds, &backgrounds,
-		  NULL);
-    
+                  XmNrows, &rows,
+                  XmNcolumns, &columns,
+                  NULL);
+
     for (c = 0; c < columns; c++) {
-	columnShadows[c] = cellShadowType;
+        XbaeMatrixSetColumnShadow(w, c, 0);
     }
 
     for (r = 0; r < rows; r++) {
-	rowShadows[r] = cellShadowType;
+        XbaeMatrixSetRowShadow(w, r, 0);
     }
     
     for (r = 0; r < rows; r++) {
-	for (c = 0; c < columns; c++) {
-	    cellShadows[r][c] = 0;
-	    backgrounds[r][c] = XmUNSPECIFIED_PIXEL;
-	}
+        for (c = 0; c < columns; c++) {
+            XbaeMatrixSetCellShadow(w, r, c, 0);
+            XbaeMatrixSetCellBackground(w, r, c, XmUNSPECIFIED_PIXEL);
+        }
     }
-    
-    XtVaSetValues(w, 
-                  XmNcellShadowTypes, cellShadows, 
-		  XmNcellBackgrounds, backgrounds,
-		  NULL);
-    
-    for(r = 0; r < rows; r++) {
-        XtFree((XtPointer) cellShadows[r]);
-        XtFree((XtPointer) backgrounds[r]);
-    }
-    XtFree((XtPointer) cellShadows);
-    XtFree((XtPointer) backgrounds);
 }
 
 void
 ArmColumn(Widget w, int column, SelectModeStruct *sms)
 {
-    int rows, columns, r;
-    unsigned char *columnShadows;
+    int rows, columns;
     unsigned char cellShadowType;
-    Pixel **backgrounds;
     
     XtVaGetValues(w,
-		  XmNrows, &rows,
-		  XmNcolumns, &columns,
-		  XmNcellShadowType, &cellShadowType,
-		  XmNcolumnShadowTypes, &columnShadows,
-		  XmNcellBackgrounds, &backgrounds,
-		  NULL);
+                  XmNrows, &rows,
+                  XmNcolumns, &columns,
+                  XmNcellShadowType, &cellShadowType,
+                  NULL);
 
     switch (cellShadowType)
     {
-	case XmSHADOW_IN:
-	    columnShadows[column] = XmSHADOW_OUT;
-	    break;
+    case XmSHADOW_IN:
+        XbaeMatrixSetColumnShadow(w, column, XmSHADOW_OUT);
+        break;
 
-	case XmSHADOW_OUT:
-	    columnShadows[column] = XmSHADOW_IN;
-	    break;
+    case XmSHADOW_OUT:
+        XbaeMatrixSetColumnShadow(w, column, XmSHADOW_IN);
+        break;
 
-	case XmSHADOW_ETCHED_IN:
-	    columnShadows[column] = XmSHADOW_ETCHED_OUT;
-	    break;
+    case XmSHADOW_ETCHED_IN:
+        XbaeMatrixSetColumnShadow(w, column, XmSHADOW_ETCHED_OUT);
+        break;
 
-	case XmSHADOW_ETCHED_OUT:
-	    columnShadows[column] = XmSHADOW_ETCHED_IN;
-	    break;
+    case XmSHADOW_ETCHED_OUT:
+        XbaeMatrixSetColumnShadow(w, column, XmSHADOW_ETCHED_IN);
+        break;
     }
 
-    for (r = 0; r < rows; r++)
-    {
-    	backgrounds[r][column] = sms->armColor;
-    }
-
-    XtVaSetValues(w, XmNcellBackgrounds, backgrounds, NULL);
-
-    /*
-     * Tobias: This is needed because we have modified the 
-     * internal state of the matrix when playing with the columnShadow
-     */
-    XbaeMatrixRefreshColumn(w, column);
-
-    for(r = 0; r < rows; r++) {
-        XtFree((XtPointer) backgrounds[r]);
-    }
-    XtFree((XtPointer) backgrounds);
+    XbaeMatrixSetColumnBackgrounds(w, column, &sms->armColor, 1);
 }
 
 void
 ArmRow(Widget w, int row, SelectModeStruct *sms)
 {
-    int rows, columns, r, c;
-    unsigned char *rowShadows;
+    int rows, columns;
     unsigned char cellShadowType;
-    Pixel **backgrounds;
     
     XtVaGetValues(w,
-		  XmNrows, &rows,
-		  XmNcolumns, &columns,
-		  XmNcellShadowType, &cellShadowType,
-		  XmNrowShadowTypes, &rowShadows,
-		  XmNcellBackgrounds, &backgrounds,
-		  NULL);
+                  XmNrows, &rows,
+                  XmNcolumns, &columns,
+                  XmNcellShadowType, &cellShadowType,
+                  NULL);
 
     switch (cellShadowType)
     {
-	case XmSHADOW_IN:
-	    rowShadows[row] = XmSHADOW_OUT;
-	    break;
+    case XmSHADOW_IN:
+        XbaeMatrixSetRowShadow(w, row, XmSHADOW_OUT);
+        break;
 
-	case XmSHADOW_OUT:
-	    rowShadows[row] = XmSHADOW_IN;
-	    break;
+    case XmSHADOW_OUT:
+        XbaeMatrixSetRowShadow(w, row, XmSHADOW_IN);
+        break;
 
-	case XmSHADOW_ETCHED_IN:
-	    rowShadows[row] = XmSHADOW_ETCHED_OUT;
-	    break;
+    case XmSHADOW_ETCHED_IN:
+        XbaeMatrixSetRowShadow(w, row, XmSHADOW_ETCHED_OUT);
+        break;
 
-	case XmSHADOW_ETCHED_OUT:
-	    rowShadows[row] = XmSHADOW_ETCHED_IN;
-	    break;
+    case XmSHADOW_ETCHED_OUT:
+        XbaeMatrixSetRowShadow(w, row, XmSHADOW_ETCHED_IN);
+        break;
     }
 
-    for (c = 0; c < columns; c++)
-    {
-	backgrounds[row][c] = sms->armColor;
-    }
-
-    XtVaSetValues(w, XmNcellBackgrounds, backgrounds, NULL);
-
-    /*
-     * Tobias: This is needed because we have modified the 
-     * internal state of the matrix when playing with the rowShadow
-     */
-    XbaeMatrixRefreshRow(w, row);
-
-    for(r = 0; r < rows; r++) {
-        XtFree((XtPointer) backgrounds[r]);
-    }
-    XtFree((XtPointer) backgrounds);
+    XbaeMatrixSetRowBackgrounds(w, row, &sms->armColor, 1);
 }
 
 void
 ArmCell(Widget w, int row, int column, SelectModeStruct *sms)
 {
     int rows, columns, r, c;
-    unsigned char *rowShadows, *columnShadows, **cellShadows;
-    unsigned char grid, cellShadowType;
-    Pixel **backgrounds;
+    unsigned char cellShadowType;
 
     XtVaGetValues(w,
-		  XmNrows, &rows,
-		  XmNcolumns, &columns,
-		  XmNgridType, &grid,
-		  XmNcellShadowType, &cellShadowType,
-		  XmNcellShadowTypes, &cellShadows,
-		  XmNrowShadowTypes, &rowShadows,
-		  XmNcolumnShadowTypes, &columnShadows,
-		  XmNcellBackgrounds, &backgrounds,
-		  NULL);
+                  XmNrows, &rows,
+                  XmNcolumns, &columns,
+                  XmNcellShadowType, &cellShadowType,
+                  NULL);
 
     for (r = 0; r < rows; r++)
     {
-	backgrounds[r][column] = sms->armColor;
-	switch (cellShadowType)
-	{
-	    case XmSHADOW_IN:
-		cellShadows[r][column] = XmSHADOW_OUT;
-		break;
+        XbaeMatrixSetCellBackground(w, r, column, sms->armColor);
+        switch (cellShadowType)
+        {
+        case XmSHADOW_IN:
+            XbaeMatrixSetCellShadow(w, r, column, XmSHADOW_OUT);
+            break;
 
-	    case XmSHADOW_OUT:
-		cellShadows[r][column] = XmSHADOW_IN;
-		break;
+        case XmSHADOW_OUT:
+            XbaeMatrixSetCellShadow(w, r, column, XmSHADOW_IN);
+            break;
 
-	    case XmSHADOW_ETCHED_IN:
-		cellShadows[r][column] = XmSHADOW_ETCHED_OUT;
-		break;
+        case XmSHADOW_ETCHED_IN:
+            XbaeMatrixSetCellShadow(w, r, column, XmSHADOW_ETCHED_OUT);
+            break;
 
-	    case XmSHADOW_ETCHED_OUT:
-		cellShadows[r][column] = XmSHADOW_ETCHED_IN;
-		break;
-	}
+        case XmSHADOW_ETCHED_OUT:
+            XbaeMatrixSetCellShadow(w, r, column, XmSHADOW_ETCHED_IN);
+            break;
+        }
     }
 
     for (c = 0; c < columns; c++)
     {
-	backgrounds[row][c] = sms->armColor;
-	switch (cellShadowType)
-	{
-	    case XmSHADOW_IN:
-		cellShadows[row][c] = XmSHADOW_OUT;
-		break;
+        XbaeMatrixSetCellBackground(w, row, c, sms->armColor);
+        switch (cellShadowType)
+        {
+        case XmSHADOW_IN:
+            XbaeMatrixSetCellShadow(w, row, c, XmSHADOW_OUT);
+            break;
 
-	    case XmSHADOW_OUT:
-		cellShadows[row][c] = XmSHADOW_IN;
-		break;
+        case XmSHADOW_OUT:
+            XbaeMatrixSetCellShadow(w, row, c, XmSHADOW_IN);
+            break;
 
-	    case XmSHADOW_ETCHED_IN:
-		cellShadows[row][c] = XmSHADOW_ETCHED_OUT;
-		break;
+        case XmSHADOW_ETCHED_IN:
+            XbaeMatrixSetCellShadow(w, row, c, XmSHADOW_ETCHED_OUT);
+            break;
 
-	    case XmSHADOW_ETCHED_OUT:
-		cellShadows[row][c] = XmSHADOW_ETCHED_IN;
-		break;
-	}
+        case XmSHADOW_ETCHED_OUT:
+            XbaeMatrixSetCellShadow(w, row, c, XmSHADOW_ETCHED_IN);
+            break;
+        }
     }
-    
-    XtVaSetValues(w, 
-                  XmNcellShadowTypes, cellShadows, 
-		  XmNcellBackgrounds, backgrounds,
-		  NULL);
-
-    for(r = 0; r < rows; r++) {
-        XtFree((XtPointer) cellShadows[r]);
-        XtFree((XtPointer) backgrounds[r]);
-    }
-    XtFree((XtPointer) cellShadows);
-    XtFree((XtPointer) backgrounds);
 }
 
 void
@@ -362,52 +322,55 @@ cbSelect(Widget w, XtPointer client, XtPointer call)
     char buf[200];
     Widget text = (Widget) client;
     XbaeMatrixSelectCellCallbackStruct *cbs =
-	(XbaeMatrixSelectCellCallbackStruct*) call;
+        (XbaeMatrixSelectCellCallbackStruct*) call;
     SelectModeStruct *sms;
     unsigned char grid;
 
-    XtVaGetValues(w, XmNuserData, &sms,XmNgridType, &grid, NULL);
+    XtVaGetValues(w, 
+                  XmNuserData, &sms,
+                  XmNgridType, &grid, 
+                  NULL);
+
     XbaeMatrixEventToXY(w, cbs->event, &x, &y);
     
     sprintf(buf, "Selected cell %d,%d : %d, %d : event %d, %d",
-	    cbs->row, cbs->column, x, y,
-	    cbs->event->xbutton.x,
-	    cbs->event->xbutton.y);
+        cbs->row, cbs->column, x, y,
+        cbs->event->xbutton.x,
+        cbs->event->xbutton.y);
     
-    if( cbs->row >= 0 && cbs->column >= 0 )
-    {
-	switch (sms->value)
-	{
-	    case SelectSelect:
-                XbaeMatrixDeselectAll( w );
-		XbaeMatrixSelectColumn( w, cbs->column );
-		XbaeMatrixSelectRow( w, cbs->row );
-		break;
+    if (cbs->row >= 0 && cbs->column >= 0 ) {
+        switch (sms->value)
+        {
+        case SelectSelect:
+            XbaeMatrixDeselectAll( w );
+            XbaeMatrixSelectColumn( w, cbs->column );
+            XbaeMatrixSelectRow( w, cbs->row );
+            break;
 
-	    case SelectHighlight:
-                XbaeMatrixUnhighlightAll( w );
-		XbaeMatrixHighlightColumn( w, cbs->column );
-		XbaeMatrixHighlightRow( w, cbs->row );
-		break;
+        case SelectHighlight:
+            XbaeMatrixUnhighlightAll( w );
+            XbaeMatrixHighlightColumn( w, cbs->column );
+            XbaeMatrixHighlightRow( w, cbs->row );
+            break;
 
-	    case SelectArm:
-                UnarmAll( w );
-		switch (grid)
-		{
-		    case XmGRID_ROW_SHADOW:
-			ArmRow( w, cbs->row, sms );
-			break;
+        case SelectArm:
+            UnarmAll( w );
+            switch (grid)
+            {
+            case XmGRID_ROW_SHADOW:
+                ArmRow( w, cbs->row, sms );
+                break;
 
-		    case XmGRID_COLUMN_SHADOW:
-			ArmColumn( w, cbs->column, sms );
-			break;
-                        
-		    default:
-			ArmCell( w, cbs->row, cbs->column, sms );
-			break;
-		}
-		break;
-	}
+            case XmGRID_COLUMN_SHADOW:
+                ArmColumn( w, cbs->column, sms );
+                break;
+
+            default:
+                ArmCell( w, cbs->row, cbs->column, sms );
+                break;
+            }
+            break;
+        }
     }
 
     XmTextSetString(text, buf);
@@ -417,73 +380,15 @@ void
 cbSelectMode(Widget w, XtPointer client, XtPointer call)
 {
     SelectModeStruct *sms;
-    XmToggleButtonCallbackStruct *cbs = (XmToggleButtonCallbackStruct *) call;
     SetValueStruct *svs = (SetValueStruct *) client;
-    unsigned char *rowShadows = NULL;
-    unsigned char *columnShadows = NULL;
-    unsigned char **cellShadows = NULL;
-    unsigned char cellShadowType;
-    Pixel **backgrounds = NULL;
-    int rows, columns, r, c;
-    static Boolean once = False;
 
     XtVaGetValues(svs->mw,
-		  XmNuserData, &sms,
-		  XmNrows, &rows,
-		  XmNcolumns, &columns,
-		  XmNcellShadowType, &cellShadowType,
-		  NULL);
+                  XmNuserData, &sms,
+                  NULL);
     
-    if (cbs->set) {
-	sms->value = svs->value;
-        if (!once && sms->value == SelectArm) {
-            /* 
-             * We need to do this once so that the matrix has these things set
-             * and a GetValues on them won't return NULL
-             */
-            once = True;
-            
-	    cellShadows = (unsigned char **) XtMalloc(rows * sizeof(unsigned char *));
-	    backgrounds = (Pixel **) XtMalloc(rows * sizeof(Pixel *));
-	    for (r = 0; r < rows; r++) {
-	        cellShadows[r] = (unsigned char*) XtMalloc(columns * sizeof(unsigned char));
-	        backgrounds[r] = (Pixel *) XtMalloc(columns * sizeof(Pixel));
-	        for (c = 0; c < columns; c++) {
-		    cellShadows[r][c] = 0;
-		    backgrounds[r][c] = XmUNSPECIFIED_PIXEL;
-	        }
-	    }
-
-	    rowShadows = (unsigned char*) XtMalloc(rows * sizeof(unsigned char));
-            for (r = 0; r < rows; r++) {
-	        rowShadows[r] = cellShadowType;
-            }
-
-	    columnShadows = (unsigned char*) XtMalloc(columns * sizeof(unsigned char));
-	    for (c = 0; c < columns; c++) {
-		    columnShadows[c] = cellShadowType;
-            }
-
-            XtVaSetValues(svs->mw,
-		          XmNcellBackgrounds, backgrounds,
-		          XmNcellShadowTypes, cellShadows,
-		          XmNrowShadowTypes, rowShadows,
-		          XmNcolumnShadowTypes, columnShadows,
-		          NULL);
-
-            XbaeMatrixRefresh(svs->mw);
-
-	    for (r = 0; r < rows; r++) {
-                XtFree((XtPointer) cellShadows[r]);
-                XtFree((XtPointer) backgrounds[r]);
-            }
-            XtFree((XtPointer) cellShadows);
-            XtFree((XtPointer) backgrounds);
-            XtFree((XtPointer) rowShadows);
-            XtFree((XtPointer) columnShadows);
-        }
-    } else {
-        switch(sms->value){
+    if (svs->value != sms->value) {
+        switch(sms->value)
+        {
         case SelectSelect:
             XbaeMatrixDeselectAll( svs->mw );
             break;
@@ -494,13 +399,15 @@ cbSelectMode(Widget w, XtPointer client, XtPointer call)
             UnarmAll(svs->mw);
             break;
         }
+
+        sms->value = svs->value;
     }
 }
 
-Widget
+void
 createSelectMode(Widget parent, Widget mw)
 {
-    Widget frame, frame2, label, rc, button;
+    Widget frame, frame2, label, option, menu, but, sb;
     SetValueStruct *svs;
     
     frame = XtVaCreateManagedWidget(
@@ -519,43 +426,37 @@ createSelectMode(Widget parent, Widget mw)
 	"Select Mode (not a resource)", xmLabelWidgetClass, frame2,
 	NULL);
     
-    rc = XtVaCreateManagedWidget(
-	"rc", xmRowColumnWidgetClass, frame,
-	XmNchildType,	    XmFRAME_WORKAREA_CHILD,
-	XmNadjustLast,	    False,
-	XmNradioBehavior,   True,
-	XmNradioAlwaysOne,  True,
-	XmNnumColumns,	    1,
-        XmNorientation,     XmHORIZONTAL,
-        XmNpacking,	    XmPACK_COLUMN,
-	NULL);
-
-    button = XtVaCreateManagedWidget(
-	"Select", xmToggleButtonWidgetClass, rc,
-	XmNset, True,
-	NULL);
+    menu = XmCreatePulldownMenu(frame, "menu", NULL, 0);
+    
+    but = XmCreatePushButton(menu, "Select", NULL, 0);
+    sb = but;
+    XtManageChild(but);
     svs = XtNew(SetValueStruct);
     svs->mw = mw;
     svs->value = SelectSelect;
-    XtAddCallback(button, XmNvalueChangedCallback, cbSelectMode, (XtPointer) svs);
+    XtAddCallback(but, XmNactivateCallback, cbSelectMode, (XtPointer) svs);
     
-    button = XtVaCreateManagedWidget(
-	"Highlight", xmToggleButtonWidgetClass, rc,
-	NULL);
+    but = XmCreatePushButton(menu, "Highlight", NULL, 0);
+    XtManageChild(but);
     svs = XtNew(SetValueStruct);
     svs->mw = mw;
     svs->value = SelectHighlight;
-    XtAddCallback(button, XmNvalueChangedCallback, cbSelectMode, (XtPointer) svs);
+    XtAddCallback(but, XmNactivateCallback, cbSelectMode, (XtPointer) svs);
     
-    button = XtVaCreateManagedWidget(
-	"Arm", xmToggleButtonWidgetClass, rc,
-	NULL);
+    but = XmCreatePushButton(menu, "Arm", NULL, 0);
+    XtManageChild(but);
     svs = XtNew(SetValueStruct);
     svs->mw = mw;
     svs->value = SelectArm;
-    XtAddCallback(button, XmNvalueChangedCallback, cbSelectMode, (XtPointer) svs);
+    XtAddCallback(but, XmNactivateCallback, cbSelectMode, (XtPointer) svs);
     
-    return frame;
+    option = XtVaCreateManagedWidget(
+    	"option", xmRowColumnWidgetClass, frame,
+        XmNisHomogeneous, True,
+        XmNrowColumnType, XmMENU_OPTION,
+        XmNsubMenuId, menu,
+        XmNmenuHistory, sb,
+        NULL);
 }
 
 /* ================================================================================================
@@ -592,21 +493,23 @@ createScaleBox(Widget mw, Widget parent, char *name, char *resource, int min, in
     label = XtVaCreateManagedWidget(
 	name, xmLabelWidgetClass, frame2,
 	NULL);
-    
+
     XtVaGetValues(mw, resource, &value, NULL);
 
     scale = XtVaCreateManagedWidget(
 	"scale", xmScaleWidgetClass, frame,
 	XmNchildType,   XmFRAME_WORKAREA_CHILD,
 	XmNorientation, XmHORIZONTAL,
+    XmNshowValue, XmNEAR_SLIDER,
 	XmNminimum,	    min,
 	XmNmaximum,	    max,
 	XmNscaleMultiple, 1,
-	XmNshowValue,   True,
-        XmNvalue, value,
+    XmNvalue, value,
 	NULL);
 
+    #if VERBOSE
     fprintf(stderr,"%s (dimension) %hd\n", resource, value);
+    #endif
 
     svs = XtNew(SetValueStruct);
     svs->mw = mw;
@@ -653,7 +556,7 @@ createToggle(Widget mw, Widget parent, char *name, char *resource)
 }
 
 /* ================================================================================================
- * The following functions are for the radio buttons
+ * The following functions are for the radio buttons (unused)
  * The resources have to be of type unsigned char or else the XtVaGetValues will do garbage
  */
 void
@@ -701,7 +604,9 @@ createButtonBox(Widget mw, Widget parent, char *name, String resource, struct na
 	NULL);
     
     XtVaGetValues(mw, resource, &value, NULL);
+    #if VERBOSE
     fprintf(stderr, "%s (unsigned char) %d\n",resource, value);
+    #endif
     
     for(p = 0; p < n_pairs; p++){
         Widget button;
@@ -722,10 +627,79 @@ createButtonBox(Widget mw, Widget parent, char *name, String resource, struct na
     return rc;
 }
 
+/* ================================================================================================
+ * The following functions are for the comboBoxes
+ * The resources have to be of type unsigned char or else the XtVaGetValues will do garbage
+ */
+
+void
+cbCombo(Widget w, XtPointer client, XtPointer call)
+{
+    SetValueStruct *svs = (SetValueStruct *) client;
+
+	XtVaSetValues(svs->mw, svs->resource, svs->value, NULL);
+}
+
+void
+createComboBox(Widget mw, Widget parent, char *name, String resource, struct name_int pairs[], int n_pairs) {
+
+    Widget frame, frame2, label, option, menu, but, sb;
+    SetValueStruct *svs;
+    int p;
+    unsigned char value;
+    
+    frame = XtVaCreateManagedWidget(
+	"frame", xmFrameWidgetClass, parent,
+	NULL);
+
+    frame2 = XtVaCreateManagedWidget(
+	"frame", xmFrameWidgetClass, frame,
+	XmNchildType,		    XmFRAME_TITLE_CHILD,
+	XmNchildHorizontalAlignment,   XmALIGNMENT_CENTER,
+	XmNchildVerticalAlignment,	    XmALIGNMENT_CENTER,
+	XmNmarginWidth,		    4,
+	NULL);
+    
+    label = XtVaCreateManagedWidget(
+	name, xmLabelWidgetClass, frame2,
+	NULL);
+
+    XtVaGetValues(mw, resource, &value, NULL);
+    #if VERBOSE
+    fprintf(stderr, "%s (unsigned char) %d\n",resource, value);
+    #endif
+
+	sb = NULL;
+    menu = XmCreatePulldownMenu(frame, "menu", NULL, 0);
+
+    for (p = 0; p < n_pairs; p++) {
+		but = XmCreatePushButton(menu, pairs[p].name, NULL, 0);
+		XtManageChild(but);
+
+        svs = XtNew(SetValueStruct);
+        svs->mw = mw;
+        svs->resource = XtNewString(resource);
+        svs->value = pairs[p].value;
+
+        XtAddCallback(but, XmNactivateCallback, cbCombo, (XtPointer) svs);
+
+        if(pairs[p].value == value) {
+            sb = but;
+        }
+    }
+
+    option = XtVaCreateManagedWidget(
+    	"option", xmRowColumnWidgetClass, frame,
+        XmNisHomogeneous, True,
+        XmNrowColumnType, XmMENU_OPTION,
+        XmNsubMenuId, menu,
+        XmNmenuHistory, sb,
+        NULL);
+}
+
 void
 createGridType(Widget parent, Widget mw)
 {
-    Widget rc;
     String resource = XtNewString(XmNgridType);
     struct name_int resource_values[] = {
             {"XmGRID_CELL_LINE", XmGRID_CELL_LINE}, 
@@ -737,12 +711,7 @@ createGridType(Widget parent, Widget mw)
             {"XmGRID_NONE" , XmGRID_NONE}
     };
 
-    rc = createButtonBox(mw, parent, "Grid Type", resource, resource_values, sizeof resource_values/sizeof *resource_values);
-    XtVaSetValues(rc,
-		  XmNorientation,   XmHORIZONTAL,
-		  XmNnumColumns,    3,
-		  XmNpacking,	    XmPACK_COLUMN,
-		  NULL);
+    createComboBox(mw, parent, "Grid Type", resource, resource_values, sizeof resource_values/sizeof *resource_values);
 }
 
 void
@@ -756,7 +725,7 @@ createCellShadowType(Widget parent, Widget mw)
             {"XmSHADOW_ETCHED_OUT", XmSHADOW_ETCHED_OUT}
     };
 
-    createButtonBox(mw, parent, "Cell Shadow Type", resource, resource_values, sizeof resource_values/sizeof *resource_values);
+    createComboBox(mw, parent, "Cell Shadow Type", resource, resource_values, sizeof resource_values/sizeof *resource_values);
 }
 
 void
@@ -770,7 +739,7 @@ createShadowType(Widget parent, Widget mw)
             {"XmSHADOW_ETCHED_OUT", XmSHADOW_ETCHED_OUT}
     };
 
-    createButtonBox(mw, parent, "Shadow Type", resource, resource_values, sizeof resource_values/sizeof *resource_values);
+    createComboBox(mw, parent, "Shadow Type", resource, resource_values, sizeof resource_values/sizeof *resource_values);
 }
 
 void
@@ -784,7 +753,7 @@ createPlacement(Widget parent, Widget mw)
             {"XmBOTTOM_RIGHT", XmBOTTOM_RIGHT}
     };
 
-    createButtonBox(mw, parent, "Scrollbar Placement", resource, resource_values, sizeof resource_values/sizeof *resource_values);
+    createComboBox(mw, parent, "Scrollbar Placement", resource, resource_values, sizeof resource_values/sizeof *resource_values);
 }
 
 void
@@ -797,7 +766,7 @@ createVerticalDisplay(Widget parent, Widget mw)
             {"XmDISPLAY_STATIC", XmDISPLAY_STATIC}
     };
 
-    createButtonBox(mw, parent, "Vertical Display Policy", resource, resource_values, sizeof resource_values/sizeof *resource_values);
+    createComboBox(mw, parent, "Vertical Display Policy", resource, resource_values, sizeof resource_values/sizeof *resource_values);
 }
 
 void
@@ -810,7 +779,33 @@ createHorizontalDisplay(Widget parent, Widget mw)
             {"XmDISPLAY_STATIC", XmDISPLAY_STATIC}
     };
 
-    createButtonBox(mw, parent, "Horizontal Display Policy", resource, resource_values, sizeof resource_values/sizeof *resource_values);
+    createComboBox(mw, parent, "Horizontal Display Policy", resource, resource_values, sizeof resource_values/sizeof *resource_values);
+}
+
+void
+createRowLabelAlignement(Widget parent, Widget mw)
+{
+    String resource = XtNewString(XmNrowLabelAlignment);
+    struct name_int resource_values[] = {
+            {"XmALIGNMENT_BEGINNING", XmALIGNMENT_BEGINNING}, 
+            {"XmALIGNMENT_CENTER", XmALIGNMENT_CENTER}, 
+            {"XmALIGNMENT_END", XmALIGNMENT_END}
+    };
+
+    createComboBox(mw, parent, "Row Label Alignment", resource, resource_values, sizeof resource_values/sizeof *resource_values);
+}
+
+void
+createWrapType(Widget parent, Widget mw)
+{
+    String resource = XtNewString(XmNwrapType);
+    struct name_int resource_values[] = {
+            {"XbaeWrapNone", XbaeWrapNone}, 
+            {"XbaeWrapContinuous", XbaeWrapContinuous}, 
+            {"XbaeWrapWord", XbaeWrapWord}
+    };
+
+    createComboBox(mw, parent, "Wrap Type", resource, resource_values, sizeof resource_values/sizeof *resource_values);
 }
 
 /* ================================================================================================
@@ -898,13 +893,12 @@ sortCB(Widget w, XtPointer client, XtPointer call)
         }
 }
 
-
 int
 main(int argc, char *argv[])
 {
     SelectModeStruct *sms;
     Widget toplevel, form, mw, text;
-    Widget rc1,rc2,rc3,rc4,rc5;
+    Widget combos, sliders, toggles;
     XtAppContext app;
 
     toplevel = XtVaAppInitialize(&app, "Choice",
@@ -964,39 +958,30 @@ main(int argc, char *argv[])
 
     XtAddCallback(mw, XmNselectCellCallback, cbSelect, (XtPointer) text);
     
-    rc1 = XtVaCreateManagedWidget(
+    combos = XtVaCreateManagedWidget(
 	"rc", xmRowColumnWidgetClass, form,
 	XmNtopAttachment,	XmATTACH_FORM,
 	XmNleftAttachment,	XmATTACH_FORM,
 	XmNadjustLast,		True,
-	XmNorientation,		XmHORIZONTAL,
-	XmNnumColumns,		2,
-	XmNpacking,		XmPACK_TIGHT,
+	XmNorientation,		XmVERTICAL,
+	XmNnumColumns,		3,
+	XmNpacking,		XmPACK_COLUMN,
 	NULL);
 
-    createGridType(rc1, mw);
-    createVerticalDisplay(rc1, mw);
-    
-    rc2 = XtVaCreateManagedWidget(
-	"rc", xmRowColumnWidgetClass, form,
-	XmNtopWidget,		rc1,
-	XmNtopAttachment,	XmATTACH_WIDGET,
-	XmNleftAttachment,	XmATTACH_FORM,
-	XmNrightAttachment,	XmATTACH_FORM,
-	XmNadjustLast,		True,
-	XmNorientation,		XmHORIZONTAL,
-	XmNnumColumns,		4,
-	XmNpacking,		XmPACK_TIGHT,
-	NULL);
+    createGridType(combos, mw);
+    createCellShadowType(combos, mw);
+    createShadowType(combos, mw);
+    createPlacement(combos, mw);
+    createVerticalDisplay(combos, mw);
+    createHorizontalDisplay(combos, mw);
+    createRowLabelAlignement(combos, mw);
+    createWrapType(combos, mw);
 
-    createPlacement(rc2, mw);
-    createCellShadowType(rc2, mw);
-    createShadowType(rc2, mw);
-    createHorizontalDisplay(rc2, mw);
+    createSelectMode(combos, mw);
 
-    rc3 = XtVaCreateManagedWidget(
+    sliders = XtVaCreateManagedWidget(
 	"rc", xmRowColumnWidgetClass, form,
-	XmNtopWidget,		rc2,
+	XmNtopWidget,		combos,
 	XmNtopAttachment,	XmATTACH_WIDGET,
 	XmNleftAttachment,	XmATTACH_FORM,
 	XmNadjustLast,		False,
@@ -1005,62 +990,53 @@ main(int argc, char *argv[])
 	XmNpacking,		XmPACK_COLUMN,
 	NULL);
     
-    createScaleBox(mw, rc3, "Shadow Thickness", XmNshadowThickness,0, 10);
-    createScaleBox(mw, rc3, "Cell Shadow Thickness", XmNcellShadowThickness, 0, 10);
-    createScaleBox(mw, rc3, "Cell Margin Height", XmNcellMarginHeight, 0, 10);
-    createScaleBox(mw, rc3, "Fixed Rows", XmNfixedRows, 0, 4);
-    createScaleBox(mw, rc3, "Trailing Fixed Rows", XmNtrailingFixedRows, 0, 4);
+    createScaleBox(mw, sliders, "Shadow Thickness", XmNshadowThickness,0, 10);
+    createScaleBox(mw, sliders, "Cell Shadow Thickness", XmNcellShadowThickness, 0, 10);
+    createScaleBox(mw, sliders, "Cell Margin Height", XmNcellMarginHeight, 0, 10);
+    createScaleBox(mw, sliders, "Fixed Rows", XmNfixedRows, 0, 5);
+    createScaleBox(mw, sliders, "Trailing Fixed Rows", XmNtrailingFixedRows, 0, 5);
 
-    createScaleBox(mw, rc3, "Space", XmNspace, 0, 10);
-    createScaleBox(mw, rc3, "Cell Highlight Thickness", XmNcellHighlightThickness, 0, 4);
-    createScaleBox(mw, rc3, "Cell Margin Width", XmNcellMarginWidth, 0, 10);
-    createScaleBox(mw, rc3, "Fixed Columns", XmNfixedColumns, 0, 4);
-    createScaleBox(mw, rc3, "Trailing Fixed Columns", XmNtrailingFixedColumns, 0, 4);
+    createScaleBox(mw, sliders, "Space", XmNspace, 0, 10);
+    createScaleBox(mw, sliders, "Cell Highlight Thickness", XmNcellHighlightThickness, 0, 4);
+    createScaleBox(mw, sliders, "Cell Margin Width", XmNcellMarginWidth, 0, 10);
+    createScaleBox(mw, sliders, "Fixed Columns", XmNfixedColumns, 0, 5);
+    createScaleBox(mw, sliders, "Trailing Fixed Columns", XmNtrailingFixedColumns, 0, 5);
 
 
-    rc4 = XtVaCreateManagedWidget(
+    toggles = XtVaCreateManagedWidget(
 	"rc", xmRowColumnWidgetClass, form,
-	XmNtopWidget,		rc2,
+	XmNtopWidget,		combos,
 	XmNtopAttachment,	XmATTACH_WIDGET,
-        XmNleftWidget,          rc3,
+        XmNleftWidget,          sliders,
         XmNleftAttachment,      XmATTACH_WIDGET,
-	XmNrightAttachment,	XmATTACH_FORM,
 	XmNadjustLast,		False,
 	XmNorientation,		XmVERTICAL,
 	XmNnumColumns,		2,
 	XmNpacking,		XmPACK_COLUMN,
 	NULL);
     
-    createToggle(mw, rc4, "Fill", XmNfill);
-    createToggle(mw, rc4, "Horzizontal Fill", XmNhorzFill);
-    createToggle(mw, rc4, "Trailing Attached Right", XmNtrailingAttachedRight);
-    createToggle(mw, rc4, "NonFixed Detached Left", XmNnonFixedDetachedLeft);
-    createToggle(mw, rc4, "Allow Column Resize", XmNallowColumnResize);
-    createToggle(mw, rc4, "Column Width in pixels", XmNcolumnWidthInPixels);
+    createToggle(mw, toggles, "Fill", XmNfill);
+    createToggle(mw, toggles, "Horzizontal Fill", XmNhorzFill);
+    createToggle(mw, toggles, "Trailing Attached Right", XmNtrailingAttachedRight);
+    createToggle(mw, toggles, "NonFixed Detached Left", XmNnonFixedDetachedLeft);
+    createToggle(mw, toggles, "Allow Column Resize", XmNallowColumnResize);
+    createToggle(mw, toggles, "Column Width in pixels", XmNcolumnWidthInPixels);
 
-    createToggle(mw, rc4, "Reverse Select", XmNreverseSelect);
+    createToggle(mw, toggles, "Reverse Select", XmNreverseSelect);
+    createToggle(mw, toggles, "Text background is cell", XmNtextBackgroundIsCell);
 
-    createToggle(mw, rc4, "Button Labels", XmNbuttonLabels);
-    createToggle(mw, rc4, "Vertical Fill", XmNvertFill);
-    createToggle(mw, rc4, "Trailing Attached Bottom", XmNtrailingAttachedBottom);
-    createToggle(mw, rc4, "NonFixed Detached Top", XmNnonFixedDetachedTop);
-    createToggle(mw, rc4, "Allow Row Resize", XmNallowRowResize);
-    createToggle(mw, rc4, "Row Height in pixels", XmNrowHeightInPixels);
+    createToggle(mw, toggles, "Button Labels", XmNbuttonLabels);
+    createToggle(mw, toggles, "Vertical Fill", XmNvertFill);
+    createToggle(mw, toggles, "Trailing Attached Bottom", XmNtrailingAttachedBottom);
+    createToggle(mw, toggles, "NonFixed Detached Top", XmNnonFixedDetachedTop);
+    createToggle(mw, toggles, "Allow Row Resize", XmNallowRowResize);
+    createToggle(mw, toggles, "Row Height in pixels", XmNrowHeightInPixels);
 
-    createToggle(mw, rc4, "Show arrows", XmNshowArrows);
-
-
-    rc5 = createSelectMode(form, mw);
-    XtVaSetValues(rc5, 	
-        XmNtopWidget,		rc4,
-	XmNtopAttachment,	XmATTACH_WIDGET,
-        XmNleftWidget,          rc3,
-        XmNleftAttachment,      XmATTACH_WIDGET,
-	XmNrightAttachment,	XmATTACH_NONE,
-        NULL);
+    createToggle(mw, toggles, "Show arrows", XmNshowArrows);
+    createToggle(mw, toggles, "Multi Line Cell", XmNmultiLineCell);
 
     LoadMatrix(mw);
-    
+
     {
         int row, column;
         int n_rows = XbaeMatrixNumRows(mw);
@@ -1078,14 +1054,17 @@ main(int argc, char *argv[])
     }
 
     XtVaSetValues(mw,
-		  XmNtopWidget,		rc3,
+		  XmNtopWidget,		sliders,
 		  XmNtopAttachment,	XmATTACH_WIDGET,
 		  XmNrightAttachment,	XmATTACH_FORM,
 		  NULL);
-		  
+
     XtManageChild(form);
+
     XtRealizeWidget(toplevel);
+        
     XtAppMainLoop(app);
     /*NOTREACHED*/
     return 0;
 }
+
