@@ -22,7 +22,7 @@
  * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELAT-
  * ING TO THE SOFTWARE.
  *
- * $Id: Actions.c,v 1.75 2004/12/07 23:55:54 tobiasoed Exp $
+ * $Id: Actions.c,v 1.95 2005/06/13 15:22:44 tobiasoed Exp $
  */
 
 #define	VERBOSE_SLIDE
@@ -35,6 +35,7 @@
 #include <XbaeConfig.h>
 #endif
 
+#include <stdlib.h>
 #include <stdio.h>
 #include <assert.h>
 
@@ -109,7 +110,6 @@ static void updateScroll(XtPointer);
 static void checkScrollValues(Widget, XtPointer, XEvent *, Boolean *);
 static void callSelectCellCallbacks(XbaeMatrixWidget, XEvent *, int, int, String *, Cardinal);
 
-static Boolean did_move = False;
 static int last_row = -1;
 static int last_column = -1;
 
@@ -149,10 +149,6 @@ void xbaeDefaultActionACT(Widget w, XEvent * event, String * params, Cardinal *
         if (!xbaeMatrixXYToRowCol(mw, &x, &y, &row, &column))
                 return;
 
-        #if 0 /* Tobias: wrong fix submitted with bug report [ 598708 ] examples/list/list2 */
-        XmProcessTraversal((Widget) mw, XmTRAVERSE_CURRENT);
-        #endif 
-        
         if (DoubleClick(mw, event, row, column)) {
                 XbaeMatrixDefaultActionCallbackStruct call_data;
 
@@ -438,14 +434,14 @@ static void Slide(Widget w, XtPointer data, XEvent * event, Boolean * cont)
                 
                 if (rd->resize_row) {
                         MoveSlide(rd->mw, motionEvent->y, &rd->currenty, &rd->rowHeights[rd->row], 
-                                  rd->mw->matrix.row_height_in_pixels, FONT_HEIGHT(rd->mw), ROW_MINIMUM_HEIGHT,
+                                  rd->mw->matrix.row_height_in_pixels, TEXT_HEIGHT(rd->mw), ROW_MINIMUM_HEIGHT,
                                   DrawSlideRow);
 
                 }
 
                 if (rd->resize_column) {
                         MoveSlide(rd->mw, motionEvent->x, &rd->currentx, &rd->columnWidths[rd->column], 
-                                  rd->mw->matrix.column_width_in_pixels, FONT_WIDTH(rd->mw), COLUMN_MINIMUM_WIDTH,
+                                  rd->mw->matrix.column_width_in_pixels, CELL_FONT_WIDTH(rd->mw), COLUMN_MINIMUM_WIDTH,
                                   DrawSlideColumn);
                 }
 
@@ -557,13 +553,14 @@ void xbaeResizeColumnsACT(Widget w, XEvent * event, String * params, Cardinal *
                  * Make it here and it's time to start the fun stuff!
                  */
 
-                Boolean haveHSB, haveVSB, resized_rows, resized_columns;
+                Boolean haveText, haveHSB, haveVSB, resized_rows, resized_columns;
 
-                if (XtIsManaged(TextChild(mw))) {
+                haveText = mw->matrix.text_child_is_mapped;
+                if (haveText) {
                         /* 
                          * Say goodbye to the TextChild -> it only gets in the way!
                          */
-                        XtSetMappedWhenManaged(TextChild(mw), False);
+                        xbaeHideTextChild(mw);
                         /*
                          * Redraw the cell that had the text field in it or it might stay blank
                          */
@@ -579,7 +576,7 @@ void xbaeResizeColumnsACT(Widget w, XEvent * event, String * params, Cardinal *
 
                 gcmask = GCForeground | GCBackground | GCFunction | GCPlaneMask | GCLineWidth;
                 XGetGCValues(display, mw->matrix.draw_gc, gcmask, &save);
-                values.foreground = mw->manager.top_shadow_color;
+                values.foreground = mw->core.background_pixel;
                 values.background = mw->manager.foreground;
                 values.function = GXxor;
                 values.plane_mask = AllPlanes;
@@ -751,9 +748,8 @@ void xbaeResizeColumnsACT(Widget w, XEvent * event, String * params, Cardinal *
                         XtFree((XtPointer) resizeData.columnWidths);
                 }
 
-                if (XtIsManaged(TextChild(mw))) {
-                        XtSetMappedWhenManaged(TextChild(mw), True);
-                        xbaeRefocusTextChild(mw);
+                if (haveText) {
+                        xbaePositionTextChild(mw);
                 }
         }
 }
@@ -764,6 +760,8 @@ void xbaeResizeColumnsACT(Widget w, XEvent * event, String * params, Cardinal *
 void xbaeProcessDragACT(Widget w, XEvent * event, String * params, Cardinal * nparams)
 {
         XbaeMatrixWidget mw;
+        XbaeMatrixCellValuesStruct cell_values;
+
         int x, y;
         int row, column;
         XbaeMatrixProcessDragCallbackStruct call_data;
@@ -792,31 +790,62 @@ void xbaeProcessDragACT(Widget w, XEvent * event, String * params, Cardinal * np
         if (!xbaeMatrixXYToRowCol(mw, &x, &y, &row, &column))
                 return;
 
+        xbaeGetCellValues(mw, row, column, False, &cell_values);
+
+        call_data.type = cell_values.drawCB.type;
+        call_data.string = cell_values.drawCB.string;
+        call_data.pixmap = cell_values.drawCB.pixmap;
+        call_data.mask = cell_values.drawCB.mask;
+        
         call_data.reason = XbaeProcessDragReason;
         call_data.event = event;
         call_data.row = row;
         call_data.column = column;
-
-        if (mw->matrix.draw_cell_callback) {
-                Pixel bgcolor, fgcolor;
-                int width, height, depth;
-
-                call_data.type =
-                    xbaeGetDrawCellValue(mw, row, column, &call_data.string, &call_data.pixmap,
-                                         &call_data.mask, &width, &height, &bgcolor, &fgcolor,
-                                         &depth);
-        } else
-                call_data.string = xbaeGetCellStringValue(mw, row, column);
-
         call_data.num_params = *nparams;
         call_data.params = params;
 
-        XtCallCallbackList((Widget) mw, mw->matrix.process_drag_callback, (XtPointer) & call_data);
+        XtCallCallbackList((Widget) mw, mw->matrix.process_drag_callback, (XtPointer) &call_data);
+        
+        if ((cell_values.drawCB.type & XbaeStringFree) == XbaeStringFree) {
+                XtFree((XtPointer) cell_values.drawCB.string);
+        }
 }
 
 /*
  * Action to edit a non-fixed cell.
  */
+static void
+callTraverseCellCallbacks(XbaeMatrixWidget mw, XEvent * event, int *row, int *column, String param,
+                        XrmQuark qparam)
+{
+        /*
+         * Call the traverseCellCallback to allow the application to
+         * perform custom traversal.
+         */
+        XbaeMatrixTraverseCellCallbackStruct call_data;
+
+        call_data.reason = XbaeTraverseCellReason;
+        call_data.event = event;
+        call_data.row = mw->matrix.current_row;
+        call_data.column = mw->matrix.current_column;
+        call_data.next_row = *row;
+        call_data.next_column = *column;
+        call_data.fixed_rows = mw->matrix.fixed_rows;
+        call_data.fixed_columns = mw->matrix.fixed_columns;
+        call_data.trailing_fixed_rows = mw->matrix.trailing_fixed_rows;
+        call_data.trailing_fixed_columns = mw->matrix.trailing_fixed_columns;
+        call_data.num_rows = mw->matrix.rows;
+        call_data.num_columns = mw->matrix.columns;
+        call_data.param = param;
+        call_data.qparam = qparam;
+
+        XtCallCallbackList((Widget) mw, mw->matrix.traverse_cell_callback,
+                           (XtPointer) & call_data);
+
+        *row = call_data.next_row;
+        *column = call_data.next_column;
+}
+
 
 void xbaeEditCellACT(Widget w, XEvent * event, String * params, Cardinal * nparams)
 {
@@ -851,7 +880,7 @@ void xbaeEditCellACT(Widget w, XEvent * event, String * params, Cardinal * npara
                                 NULL, 0);
                 return;
         }
-
+        
         /*
          * Make sure we have a single parm
          */
@@ -885,7 +914,7 @@ void xbaeEditCellACT(Widget w, XEvent * event, String * params, Cardinal * npara
                 /*
                  * Find out what cell has the focus
                  */
-                if (w == TextChild(mw) || w == ClipChild(mw)) {
+                if (w == TextChild(mw)) {
                         current_focus_row = mw->matrix.current_row;
                         current_focus_column = mw->matrix.current_column;
                 } else if (mw->matrix.per_cell) {
@@ -947,8 +976,8 @@ void xbaeEditCellACT(Widget w, XEvent * event, String * params, Cardinal * npara
                          */
                         if (!mw->matrix.traverse_fixed) {
                                 /* check scrollable boundary */
-                                if (current_focus_row != TRAILING_VERT_ORIGIN(mw) - 1
-                                    || current_focus_column != TRAILING_HORIZ_ORIGIN(mw) - 1) {
+                                if (current_focus_row != TRAILING_ROW_ORIGIN(mw) - 1
+                                    || current_focus_column != TRAILING_COLUMN_ORIGIN(mw) - 1) {
                                         column++;
                                         if (IS_TRAILING_FIXED_COLUMN(mw, column)) {
                                                 column = mw->matrix.fixed_columns;
@@ -979,7 +1008,7 @@ void xbaeEditCellACT(Widget w, XEvent * event, String * params, Cardinal * npara
                                     || current_focus_column != mw->matrix.fixed_columns) {
                                         column--;
                                         if (IS_LEADING_FIXED_COLUMN(mw, column)) {
-                                                column = TRAILING_HORIZ_ORIGIN(mw) - 1;
+                                                column = TRAILING_COLUMN_ORIGIN(mw) - 1;
                                                 row--;
                                         }
                                 }
@@ -1008,7 +1037,7 @@ void xbaeEditCellACT(Widget w, XEvent * event, String * params, Cardinal * npara
 
                         if (!mw->matrix.traverse_fixed) {
                                 if (IS_LEADING_FIXED_ROW(mw, row))
-                                        row = TRAILING_VERT_ORIGIN(mw) - 1;
+                                        row = TRAILING_ROW_ORIGIN(mw) - 1;
                         } else {
                                 if (row < 0)
                                         row = mw->matrix.rows - 1;
@@ -1021,40 +1050,12 @@ void xbaeEditCellACT(Widget w, XEvent * event, String * params, Cardinal * npara
          * perform custom traversal.
          */
         if (mw->matrix.traverse_cell_callback) {
-                XbaeMatrixTraverseCellCallbackStruct call_data;
-
-                call_data.reason = XbaeTraverseCellReason;
-                call_data.event = event;
-                call_data.row = mw->matrix.current_row;
-                call_data.column = mw->matrix.current_column;
-                call_data.next_row = row;
-                call_data.next_column = column;
-                call_data.fixed_rows = mw->matrix.fixed_rows;
-                call_data.fixed_columns = mw->matrix.fixed_columns;
-                call_data.trailing_fixed_rows = mw->matrix.trailing_fixed_rows;
-                call_data.trailing_fixed_columns = mw->matrix.trailing_fixed_columns;
-                call_data.num_rows = mw->matrix.rows;
-                call_data.num_columns = mw->matrix.columns;
-                call_data.param = params[0];
-                call_data.qparam = q;
-
-                XtCallCallbackList((Widget) mw, mw->matrix.traverse_cell_callback,
-                                   (XtPointer) & call_data);
-
-                row = call_data.next_row;
-                column = call_data.next_column;
+                callTraverseCellCallbacks(mw, event, &row, &column, params[0], q);
         }
 
-        /*
-         * Attempt to edit the new cell using the edit_cell method.
-         * If we are editing a cell based on pointer position, we always
-         * call edit_cell.  Otherwise, we must be editing a new cell to
-         * call edit_cell.
-         */
-        if (q == QPointer || (row != current_focus_row || column != current_focus_column))
-                ((XbaeMatrixWidgetClass) XtClass(mw))->matrix_class.edit_cell(mw, event, row,
-                                                                                  column, params,
-                                                                                  *nparams);
+        ((XbaeMatrixWidgetClass) XtClass(mw))->matrix_class.edit_cell(mw, event, 
+                                                                      row, column, 
+                                                                      params, *nparams);
 }
 
 /*
@@ -1106,10 +1107,18 @@ void xbaeCancelEditACT(Widget w, XEvent * event, String * params, Cardinal * npa
                                 NULL, 0);
                 return;
         }
-        /*
-         * Call the cancel_edit method
-         */
-        ((XbaeMatrixWidgetClass) XtClass(mw))->matrix_class.cancel_edit(mw, unmap);
+        
+        if (!mw->matrix.text_child_is_mapped) {
+                /*
+                 * The textChild is not visible, call ManagerParentCancel
+                 */
+                XtCallActionProc((Widget) mw, "ManagerParentCancel", event, params, *nparams);
+        } else {
+                /*
+                 * The textChild is visible, call the cancel_edit method
+                 */
+                ((XbaeMatrixWidgetClass) XtClass(mw))->matrix_class.cancel_edit(mw, unmap);
+        }
 }
 
 /*
@@ -1161,7 +1170,17 @@ void xbaeCommitEditACT(Widget w, XEvent * event, String * params, Cardinal * npa
                 return;
         }
 
-        ((XbaeMatrixWidgetClass) XtClass(mw))->matrix_class.commit_edit(mw, event, unmap);
+        if (!mw->matrix.text_child_is_mapped) {
+                /*
+                 * The textChild is not visible, call ManagerParentActivate
+                 */
+                XtCallActionProc((Widget) mw, "ManagerParentActivate", event, params, *nparams);
+        } else {
+                /*
+                 * The textChild is visible, call the commit_edit method
+                 */
+                ((XbaeMatrixWidgetClass) XtClass(mw))->matrix_class.commit_edit(mw, event, unmap);
+        }
 }
 
 static int DoubleClick(XbaeMatrixWidget mw, XEvent * event, int row, int column)
@@ -1308,7 +1327,6 @@ static int DoubleClick(XbaeMatrixWidget mw, XEvent * event, int row, int column)
         in_labels = in_rows && in_columns && ((row == -1) ^ (column == -1));
 
         if (event->type == ButtonPress) {
-                did_move = False;
                 last_row = row;
                 last_column = column;
         }
@@ -1393,8 +1411,12 @@ static int DoubleClick(XbaeMatrixWidget mw, XEvent * event, int row, int column)
 }
 
 /* ARGSUSED */
-/* Obsolete */
-void xbaeTraverseNextACT(Widget w, XEvent * event, String * params, Cardinal * nparams)
+/*
+ * Dean Phillips needs xbaeTraversePrev/NextACT because ManagerGadgetNext/PrevTabGroup()
+ * cause NutCracker's Motif implementation to crash.
+ */
+void
+xbaeTraverseNextACT(Widget w, XEvent * event, String * params, Cardinal * nparams)
 {
         XbaeMatrixWidget mw;
 
@@ -1410,13 +1432,13 @@ void xbaeTraverseNextACT(Widget w, XEvent * event, String * params, Cardinal * n
                                 "XbaeMatrix: Bad widget passed to TraverseNext action", NULL, 0);
                 return;
         }
-
-        XmProcessTraversal(ClipChild(mw), XmTRAVERSE_NEXT_TAB_GROUP);
+        
+        XmProcessTraversal(TextChild(mw), XmTRAVERSE_NEXT_TAB_GROUP);
 }
 
 /* ARGSUSED */
-/* Obsolete */
-void xbaeTraversePrevACT(Widget w, XEvent * event, String * params, Cardinal * nparams)
+void
+xbaeTraversePrevACT(Widget w, XEvent * event, String * params, Cardinal * nparams)
 {
         XbaeMatrixWidget mw;
 
@@ -1433,7 +1455,7 @@ void xbaeTraversePrevACT(Widget w, XEvent * event, String * params, Cardinal * n
                 return;
         }
 
-        XmProcessTraversal(ClipChild(mw), XmTRAVERSE_PREV_TAB_GROUP);
+        XmProcessTraversal(TextChild(mw), XmTRAVERSE_PREV_TAB_GROUP);
 }
 
 static void
@@ -1511,16 +1533,11 @@ void xbaeSelectCellACT(Widget w, XEvent * event, String * params, Cardinal * num
                 String params[1] = { "extend" };
                 ss->mw->matrix.scroll_select = False;
 
-                if (!did_move) {
-                        callSelectCellCallbacks(ss->mw, ss->event, last_row, last_column, NULL, 0);
-                }
-
                 callSelectCellCallbacks(ss->mw, ss->event, new_row, new_column, params,
                                         sizeof params / sizeof *params);
 
                 ss->mw->matrix.scroll_select = old_scroll_select;
 
-                did_move = True;
                 last_row = new_row;
                 last_column = new_column;
         }
@@ -1584,14 +1601,13 @@ static void updateScroll(XtPointer data)
                         dist = NON_FIXED_ROW_POSITION(ss->mw) - ss->currenty;
                         xbaeScrollRows(ss->mw, True, Min(dist, VERT_ORIGIN(ss->mw)));
                         row = xbaeTopRow(ss->mw);
-                } else if (ss->currenty >= NON_FIXED_ROW_POSITION(ss->mw) + VISIBLE_NON_FIXED_HEIGHT(ss->mw)
-                           && VERT_ORIGIN(ss->mw) + VISIBLE_NON_FIXED_HEIGHT(ss->mw) <
-                           NON_FIXED_TOTAL_HEIGHT(ss->mw)) {
+                } else if (   ss->currenty >= NON_FIXED_ROW_POSITION(ss->mw) + VISIBLE_NON_FIXED_HEIGHT(ss->mw)
+                           && VERT_ORIGIN(ss->mw) + VISIBLE_NON_FIXED_HEIGHT(ss->mw) < NON_FIXED_HEIGHT(ss->mw)) {
                         /* bellow the non fixed cells */
                         dist = ss->currenty - (NON_FIXED_ROW_POSITION(ss->mw) + VISIBLE_NON_FIXED_HEIGHT(ss->mw) - 1);
                         xbaeScrollRows(ss->mw, False,
                                        Min(dist,
-                                           NON_FIXED_TOTAL_HEIGHT(ss->mw) - VERT_ORIGIN(ss->mw) -
+                                           NON_FIXED_HEIGHT(ss->mw) - VERT_ORIGIN(ss->mw) -
                                            VISIBLE_NON_FIXED_HEIGHT(ss->mw)));
                         row = xbaeBottomRow(ss->mw);
                 }
@@ -1601,13 +1617,12 @@ static void updateScroll(XtPointer data)
                         dist = NON_FIXED_COLUMN_POSITION(ss->mw) - ss->currentx;
                         xbaeScrollColumns(ss->mw, True, Min(dist, HORIZ_ORIGIN(ss->mw)));
                         column = xbaeLeftColumn(ss->mw);
-                } else if (ss->currentx >= NON_FIXED_COLUMN_POSITION(ss->mw) + VISIBLE_NON_FIXED_WIDTH(ss->mw)
-                           && HORIZ_ORIGIN(ss->mw) + VISIBLE_NON_FIXED_WIDTH(ss->mw) <
-                           NON_FIXED_TOTAL_WIDTH(ss->mw)) {
+                } else if (   ss->currentx >= NON_FIXED_COLUMN_POSITION(ss->mw) + VISIBLE_NON_FIXED_WIDTH(ss->mw)
+                           && HORIZ_ORIGIN(ss->mw) + VISIBLE_NON_FIXED_WIDTH(ss->mw) < NON_FIXED_WIDTH(ss->mw)) {
                         dist = ss->currentx - (NON_FIXED_COLUMN_POSITION(ss->mw) + VISIBLE_NON_FIXED_WIDTH(ss->mw) - 1);
                         xbaeScrollColumns(ss->mw, False,
                                           Min(dist,
-                                              NON_FIXED_TOTAL_WIDTH(ss->mw) - HORIZ_ORIGIN(ss->mw) -
+                                              NON_FIXED_WIDTH(ss->mw) - HORIZ_ORIGIN(ss->mw) -
                                               VISIBLE_NON_FIXED_WIDTH(ss->mw)));
                         column = xbaeRightColumn(ss->mw);
                 }
@@ -1716,6 +1731,16 @@ void xbaePageDownACT(Widget w, XEvent * event, String * params, Cardinal * npara
         XbaeMatrixWidget mw;
         char *down = "0";
         int vert_origin;
+        
+        char *param = "PageDown";
+        static XrmQuark qparam = NULLQUARK;
+        /*
+         * Get a static quarks for our parm 
+         */
+        if (qparam == NULLQUARK) {
+                qparam = XrmPermStringToQuark(param);
+        }
+        
         /*
          * Get Matrix widget and make sure it is a Matrix subclass.
          * w should be the textField widget.
@@ -1740,12 +1765,22 @@ void xbaePageDownACT(Widget w, XEvent * event, String * params, Cardinal * npara
         
         XtCallActionProc(VertScrollChild(mw), "PageDownOrRight", event, &down, 1);
 
-        if (VERT_ORIGIN(mw) != vert_origin)
+        if (VERT_ORIGIN(mw) != vert_origin) {
                 /*
                  * Position the cursor at the top most non fixed row if there was
                  * a page down
                  */
-                XbaeMatrixEditCell((Widget) mw, xbaeTopRow(mw), mw->matrix.current_column);
+                int row = xbaeTopRow(mw);
+                int column = mw->matrix.current_column < 0 ? xbaeLeftColumn(mw) : mw->matrix.current_column;
+
+                if (mw->matrix.traverse_cell_callback) {
+                        callTraverseCellCallbacks(mw, event, &row, &column, param, qparam);
+                }
+
+                ((XbaeMatrixWidgetClass) XtClass(mw))->matrix_class.edit_cell(mw, event, 
+                                                                              row, column, 
+                                                                              params, *nparams);
+        }
 }
 
 /* ARGSUSED */
@@ -1755,6 +1790,15 @@ void xbaePageUpACT(Widget w, XEvent * event, String * params, Cardinal * nparams
         char *up = "0";
         int vert_origin;
 
+        char *param = "PageUp";
+        static XrmQuark qparam = NULLQUARK;
+        /*
+         * Get a static quarks for our parm 
+         */
+        if (qparam == NULLQUARK) {
+                qparam = XrmPermStringToQuark(param);
+        }
+        
         /*
          * Get Matrix widget and make sure it is a Matrix subclass.
          * w should be the textField widget.
@@ -1779,10 +1823,88 @@ void xbaePageUpACT(Widget w, XEvent * event, String * params, Cardinal * nparams
 
         XtCallActionProc(VertScrollChild(mw), "PageUpOrLeft", event, &up, 1);
 
-        if (VERT_ORIGIN(mw) != vert_origin)
+        if (VERT_ORIGIN(mw) != vert_origin) {
                 /*
                  * Position the cursor at the top most non fixed row if there was
-                 * a page down
+                 * a page up
                  */
-                XbaeMatrixEditCell((Widget) mw, xbaeTopRow(mw), mw->matrix.current_column);
+                int row = xbaeTopRow(mw);
+                int column = mw->matrix.current_column < 0 ? xbaeLeftColumn(mw) : mw->matrix.current_column;
+
+                if (mw->matrix.traverse_cell_callback) {
+                        callTraverseCellCallbacks(mw, event, &row, &column, param, qparam);
+                }
+
+                ((XbaeMatrixWidgetClass) XtClass(mw))->matrix_class.edit_cell(mw, event, 
+                                                                              row, column, 
+                                                                              params, *nparams);
+        }
+}
+
+void xbaeScrollRowsACT(Widget w, XEvent * event, String * params, Cardinal * nparams)
+{
+        XbaeMatrixWidget mw;
+        int step;
+        String end;
+        /*
+         * Get Matrix widget and make sure it is a Matrix subclass.
+         * w should be the textField widget.
+         */
+        if (XtIsSubclass(w, xbaeMatrixWidgetClass))
+                mw = (XbaeMatrixWidget) w;
+        else if (XtIsSubclass(XtParent(w), xbaeMatrixWidgetClass))
+                mw = (XbaeMatrixWidget) XtParent(w);
+        else {
+                XtAppWarningMsg(XtWidgetToApplicationContext(w), "scrollRowsAct", "badWidget",
+                                "XbaeMatrix", "XbaeMatrix: Bad widget passed to ScrollRows action",
+                                NULL, 0);
+                return;
+        }
+    
+        if (*nparams == 1) {
+                step = strtol(params[0],&end,0);
+        }
+        
+        if (*nparams != 1 || end == params[0]) {
+                XtAppWarningMsg(XtWidgetToApplicationContext(w), "scrollRowsAct", "badWidget",
+                                "XbaeMatrix", "XbaeMatrix: Bad parameter passed to ScrollRows action",
+                                NULL, 0);
+                return;
+        }                
+    
+        xbaeScrollRows(mw, step < 0, step < 0 ? -step : step);
+}
+
+void xbaeScrollColumnsACT(Widget w, XEvent * event, String * params, Cardinal * nparams)
+{
+        XbaeMatrixWidget mw;
+        int step;
+        String end;
+        /*
+         * Get Matrix widget and make sure it is a Matrix subclass.
+         * w should be the textField widget.
+         */
+        if (XtIsSubclass(w, xbaeMatrixWidgetClass))
+                mw = (XbaeMatrixWidget) w;
+        else if (XtIsSubclass(XtParent(w), xbaeMatrixWidgetClass))
+                mw = (XbaeMatrixWidget) XtParent(w);
+        else {
+                XtAppWarningMsg(XtWidgetToApplicationContext(w), "scrollColumnsAct", "badWidget",
+                                "XbaeMatrix", "XbaeMatrix: Bad widget passed to ScrollColumns action",
+                                NULL, 0);
+                return;
+        }
+
+        if (*nparams == 1) {
+                step = strtol(params[0],&end,0);
+        }
+        
+        if (*nparams != 1 || end == params[0]) {
+                XtAppWarningMsg(XtWidgetToApplicationContext(w), "scrollColumnssAct", "badWidget",
+                                "XbaeMatrix", "XbaeMatrix: Bad parameter passed to ScrollColumns action",
+                                NULL, 0);
+                return;
+        }
+
+        xbaeScrollColumns(mw, step < 0, step < 0 ? -step : step);
 }
