@@ -24,7 +24,7 @@
  *
  * MatrixWidget Author: Andrew Wason, Bellcore, aw@bae.bellcore.com
  *
- * $Id: Utils.c,v 1.90 2004/12/07 23:55:57 tobiasoed Exp $
+ * $Id: Utils.c,v 1.112 2005/07/12 09:13:30 dannybackx Exp $
  */
 
 /*
@@ -150,7 +150,7 @@ Widget xbaeGetCellClip(XbaeMatrixWidget mw, int row, int column)
  */
 
 static void 
-xbaeGetPosition(int n, Boolean size_in_pixels, short *sizes, int *positions, int font_size, int text_offset) 
+xbaeGetPosition(int n, Boolean size_in_pixels, short *sizes, int *positions, int font_size, int border_size) 
 {
         int i, pos;
         
@@ -159,7 +159,7 @@ xbaeGetPosition(int n, Boolean size_in_pixels, short *sizes, int *positions, int
                         positions[i] = pos;
                 }
         } else {
-                for (i = 0, pos = 0; i < n; pos += sizes[i] * font_size + text_offset * 2, i++) {
+                for (i = 0, pos = 0; i < n; pos += sizes[i] * font_size + 2 * border_size, i++) {
                         positions[i] = pos;
                 }
         }
@@ -177,8 +177,8 @@ void xbaeGetColumnPositions(XbaeMatrixWidget mw)
                         mw->matrix.column_width_in_pixels,
                         mw->matrix.column_widths, 
                         mw->matrix.column_positions,
-                        FONT_WIDTH(mw),
-                        TEXT_WIDTH_OFFSET(mw));
+                        CELL_FONT_WIDTH(mw),
+                        CELL_BORDER_WIDTH(mw));
 }
 
 void xbaeGetRowPositions(XbaeMatrixWidget mw)
@@ -187,12 +187,12 @@ void xbaeGetRowPositions(XbaeMatrixWidget mw)
                         mw->matrix.row_height_in_pixels,
                         mw->matrix.row_heights, 
                         mw->matrix.row_positions,
-                        FONT_HEIGHT(mw),
-                        TEXT_HEIGHT_OFFSET(mw));
+                        TEXT_HEIGHT(mw),
+                        CELL_BORDER_HEIGHT(mw));
 }
 
 static int
-xbaeCheckPosition(int n, Boolean size_in_pixels, short *sizes, int *positions, int font_size, int text_offset, int j)
+xbaeCheckPosition(int n, Boolean size_in_pixels, short *sizes, int *positions, int font_size, int border_size, int j)
 {
         int i, pos;
 
@@ -201,7 +201,7 @@ xbaeCheckPosition(int n, Boolean size_in_pixels, short *sizes, int *positions, i
                         assert(positions[i] == pos);
                 }
         } else {
-                for (i = 0, pos = 0; i < n; pos += sizes[i] * font_size + text_offset * 2, i++) {
+                for (i = 0, pos = 0; i < n; pos += sizes[i] * font_size + 2 * border_size, i++) {
                         assert(positions[i] == pos);
                 }
         }
@@ -218,8 +218,8 @@ int xbaeCheckColumnPosition(XbaeMatrixWidget mw, int column)
                                  mw->matrix.column_width_in_pixels,
                                  mw->matrix.column_widths, 
                                  mw->matrix.column_positions,
-                                 FONT_WIDTH(mw),
-                                 TEXT_WIDTH_OFFSET(mw),
+                                 CELL_FONT_WIDTH(mw),
+                                 CELL_BORDER_WIDTH(mw),
                                  column);
 }
 
@@ -229,8 +229,8 @@ int xbaeCheckRowPosition(XbaeMatrixWidget mw, int row)
                                  mw->matrix.row_height_in_pixels,
                                  mw->matrix.row_heights, 
                                  mw->matrix.row_positions,
-                                 FONT_HEIGHT(mw),
-                                 TEXT_HEIGHT_OFFSET(mw),
+                                 TEXT_HEIGHT(mw),
+                                 CELL_BORDER_HEIGHT(mw),
                                  row);
 }
 
@@ -339,11 +339,11 @@ xbaeGetVisibleCells(XbaeMatrixWidget mw, int *top_row, int *bottom_row, int *lef
 int
 xbaeCalculateVertOrigin(XbaeMatrixWidget mw, int top_row)
 {
-        if (NON_FIXED_TOTAL_HEIGHT(mw) < VISIBLE_NON_FIXED_HEIGHT(mw)) {
+        if (NON_FIXED_HEIGHT(mw) < VISIBLE_NON_FIXED_HEIGHT(mw)) {
                 return 0;
-        } else if (ROW_POSITION(mw, TRAILING_VERT_ORIGIN(mw)) - 
+        } else if (ROW_POSITION(mw, TRAILING_ROW_ORIGIN(mw)) - 
                    ROW_POSITION(mw, mw->matrix.fixed_rows + top_row) < VISIBLE_NON_FIXED_HEIGHT(mw)) {
-               return NON_FIXED_TOTAL_HEIGHT(mw) - VISIBLE_NON_FIXED_HEIGHT(mw);
+               return NON_FIXED_HEIGHT(mw) - VISIBLE_NON_FIXED_HEIGHT(mw);
         } else {
                 return ROW_POSITION(mw, mw->matrix.fixed_rows + top_row) -
                        FIXED_ROW_HEIGHT(mw);
@@ -352,11 +352,11 @@ xbaeCalculateVertOrigin(XbaeMatrixWidget mw, int top_row)
 
 int
 xbaeCalculateHorizOrigin(XbaeMatrixWidget mw, int left_column) {
-        if (NON_FIXED_TOTAL_WIDTH(mw) < VISIBLE_NON_FIXED_WIDTH(mw)) {
+        if (NON_FIXED_WIDTH(mw) < VISIBLE_NON_FIXED_WIDTH(mw)) {
                 return 0;
-        } else if (COLUMN_POSITION(mw, TRAILING_HORIZ_ORIGIN(mw)) - 
+        } else if (COLUMN_POSITION(mw, TRAILING_COLUMN_ORIGIN(mw)) - 
                    COLUMN_POSITION(mw, mw->matrix.fixed_columns + left_column) < VISIBLE_NON_FIXED_WIDTH(mw)) {
-               return NON_FIXED_TOTAL_WIDTH(mw) - VISIBLE_NON_FIXED_WIDTH(mw);
+               return NON_FIXED_WIDTH(mw) - VISIBLE_NON_FIXED_WIDTH(mw);
         } else {
                 return COLUMN_POSITION(mw, mw->matrix.fixed_columns + left_column) -
                        FIXED_COLUMN_WIDTH(mw);
@@ -524,11 +524,9 @@ void xbaeMakeCellVisible(XbaeMatrixWidget mw, int row, int column)
 
 void xbaeComputeSize(XbaeMatrixWidget mw, Boolean compute_width, Boolean compute_height)
 {
-        unsigned long full_width = ROW_LABEL_WIDTH(mw) + 2 * mw->manager.shadow_thickness
-            + TOTAL_WIDTH(mw);
-        unsigned long full_height = COLUMN_LABEL_HEIGHT(mw) + 2 * mw->manager.shadow_thickness
-            + TOTAL_HEIGHT(mw);
-        unsigned long width, height;
+        int full_width = TOTAL_WIDTH(mw) + ROW_LABEL_WIDTH(mw) + 2 * mw->manager.shadow_thickness;
+        int full_height = TOTAL_HEIGHT(mw) + COLUMN_LABEL_HEIGHT(mw) + 2 * mw->manager.shadow_thickness;
+        int width, height;
 
         /*
          * Calculate our width.
@@ -536,15 +534,31 @@ void xbaeComputeSize(XbaeMatrixWidget mw, Boolean compute_width, Boolean compute
          * Otherwise, if the compute_width flag is set, then we are full width.
          * Otherwise we keep whatever width we are.
          */
-        if (mw->matrix.visible_columns)
+
+        DEBUGOUT(_XbaeDebug(__FILE__, (Widget) mw,
+                "xbaeComputeSize compute_width = %s   compute_height = %s\n"
+                "                visible_columns = %d visible_rows = %d\n"
+                "                width = %d           height =%d\n"
+                "                full_width = %d      full_height = %d\n",
+                compute_width ? "True" : "False",
+                compute_height ? "True" : "False",
+                mw->matrix.visible_columns,
+                mw->matrix.visible_rows,
+                mw->core.width, 
+                mw->core.height,
+                full_width,
+                full_height));
+
+        if (mw->matrix.visible_columns) {
                 width = ROW_LABEL_WIDTH(mw) + 2 * mw->manager.shadow_thickness 
                     + FIXED_COLUMN_WIDTH(mw) + TRAILING_FIXED_COLUMN_WIDTH(mw)
-                    + NON_FIXED_TOTAL_WIDTH(mw) * mw->matrix.visible_columns 
+                    + NON_FIXED_WIDTH(mw) * mw->matrix.visible_columns 
                      / (mw->matrix.columns - mw->matrix.fixed_columns - mw->matrix.trailing_fixed_columns);
-        else if (compute_width)
+        } else if (compute_width) {
                 width = full_width;
-        else
+        } else {
                 width = mw->core.width;
+        }
 
         /*
          * Calculate our height.
@@ -555,7 +569,7 @@ void xbaeComputeSize(XbaeMatrixWidget mw, Boolean compute_width, Boolean compute
         if (mw->matrix.visible_rows) {
                 height = COLUMN_LABEL_HEIGHT(mw) + 2 * mw->manager.shadow_thickness 
                     + FIXED_ROW_HEIGHT(mw) + TRAILING_FIXED_ROW_HEIGHT(mw)
-                    + NON_FIXED_TOTAL_HEIGHT(mw) * mw->matrix.visible_rows 
+                    + NON_FIXED_HEIGHT(mw) * mw->matrix.visible_rows 
                      / (mw->matrix.rows - mw->matrix.fixed_rows - mw->matrix.trailing_fixed_rows);
         } else if (compute_height) {
                 height = full_height;
@@ -563,124 +577,155 @@ void xbaeComputeSize(XbaeMatrixWidget mw, Boolean compute_width, Boolean compute
                 height = mw->core.height;
         }
 
+        /*
+         * If we are allowed to modify our height and we need to display a hsb 
+         * include it's size in the computation
+         */
+        if (   (compute_height || mw->matrix.visible_rows)
+            && (   (mw->matrix.hsb_display_policy == XmDISPLAY_STATIC)
+                || (mw->matrix.hsb_display_policy == XmDISPLAY_AS_NEEDED && width < full_width)))
+                height += HORIZ_SB_HEIGHT(mw);
+
+        /*
+         * If we are allowed to modify our width and we need to display a vsb 
+         * include it's size in the computation
+         */
+        if (   (compute_width || mw->matrix.visible_columns)
+            && (   (mw->matrix.vsb_display_policy == XmDISPLAY_STATIC)
+                || (mw->matrix.vsb_display_policy == XmDISPLAY_AS_NEEDED && height < full_height)))
+                width += VERT_SB_WIDTH(mw);
+
         /*
          * Store our calculated size.
          */
         mw->core.width = width;
         mw->core.height = height;
 
-        /*
-         * If we are less than full width or our horizontal display policy is
-         * constant, then we need an HSB, so increment our height by the size
-         * of the HSB (if we are allowed to modify our height and we are allowed
-         * to have an HSB).
-         */
-        if (((width < full_width) || (XmDISPLAY_STATIC == mw->matrix.hsb_display_policy))
-            && (compute_height || mw->matrix.visible_rows)
-            && (XmDISPLAY_NONE != mw->matrix.hsb_display_policy))
-                mw->core.height += HORIZ_SB_HEIGHT(mw);
-
-        /*
-         * If we are less than full height or our vertical display policy is
-         * constant, then we need a VSB, so increment our width by the size
-         * of the VSB (if we are allowed to modify our width and we are allowed
-         * to have a VSB).
-         */
-        if (((height < full_height) || (XmDISPLAY_STATIC == mw->matrix.vsb_display_policy))
-            && (compute_width || mw->matrix.visible_columns)
-            && (XmDISPLAY_NONE != mw->matrix.vsb_display_policy))
-                mw->core.width += VERT_SB_WIDTH(mw);
-
         /*
          * Save our calculated size for use in our query_geometry method.
          * This is the size we really want to be (not necessarily the size
          * we will end up being).
          */
-        mw->matrix.desired_width = mw->core.width;
-        mw->matrix.desired_height = mw->core.height;
+        mw->matrix.desired_width = width;
+        mw->matrix.desired_height = height;
 
         DEBUGOUT(_XbaeDebug(__FILE__, (Widget) mw,
-                            "xbaeComputeSize -> w %d h %d\n",
-                            mw->matrix.desired_width, mw->matrix.desired_height));
+                            "xbaeComputeSize -> w %d h %d\n", width, height));
 }
 
 /*
- * Return the length of the longest row label
+ * Return the length of the longest line in string
  */
-short xbaeMaxRowLabel(XbaeMatrixWidget mw)
+static int xbaeMaxLen(String string) {
+        char *nl;
+        int max_len = 0;
+
+        /*
+         * Get the length of the longest line terminated by \n
+         */
+        while ((nl = strchr(string, '\n')) != NULL) {
+                int len = nl - string;
+                if (len > max_len) {
+                    max_len = len;
+                }
+                string = nl + 1;
+        }
+
+        /* 
+         * If the last line wasn't terminated with a \n take it into account
+         */
+        if (*string != '\0') {
+                int len = strlen(string);
+                if (len > max_len) {
+                        max_len = len;
+                }
+        }
+
+        return max_len;
+}
+
+/*
+ * Return the number of lines in string
+ */
+static int xbaeCountLines(String string) {
+        char *nl;
+        int n_lines = 0;
+        
+        /*
+         * Count the number of lines terminated by \n
+         */
+        while ((nl = strchr(string, '\n')) != NULL) {
+                n_lines++;
+                string = nl + 1;
+        }
+
+        /*
+         * If the last line wasn't terminated by a \n take it into account
+         */
+        if (*string != '\0') {
+                n_lines++;
+        }
+        
+        return n_lines;
+}
+
+/*
+ * Return the length of the longest line of all row_labels
+ */
+int xbaeCalculateLabelMaxLength(XbaeMatrixWidget mw, String *labels, XmString *xmlabels, int n_labels)
 {
         int i;
-        short max = 0, len;
+        int max_len = 0;
 
         /*
          * Determine the length of the longest row label
          */
-        for (i = 0; i < mw->matrix.rows; i++) {
-                if (mw->matrix.row_labels[i]) {
-                        len = strlen(mw->matrix.row_labels[i]);
-                        if (len > max) {
-                                max = len;
+        if (labels || xmlabels) {
+                for (i = 0; i < n_labels; i++) {
+                        int len = 0;
+                        if (xmlabels && xmlabels[i]) {
+#if XmVERSION >= 2
+                                int width = XmStringWidth((mw->matrix.render_table)
+                                                           ? mw->matrix.render_table 
+                                                           : (mw->matrix.label_font_list)
+                                                              ? mw->matrix.label_font_list 
+                                                              : mw->matrix.font_list,
+                                                          xmlabels[i]);
+#else
+                                int width = XmStringWidth((mw->matrix.label_font_list)
+                                                            ? mw->matrix.label_font_list 
+                                                            : mw->matrix.font_list,
+                                                          xmlabels[i]);
+#endif
+                                len = width / LABEL_FONT_WIDTH(mw) + ((width % LABEL_FONT_WIDTH(mw)) > 0);
+                        } else if (labels && labels[i]) {
+                                len = xbaeMaxLen(labels[i]);
+                        }
+                        if (len > max_len) {
+                                max_len = len;
                         }
                 }
         }
-        return max;
+        return max_len;
 }
 
-void xbaeParseColumnLabel(String label, ColumnLabelLines label_line)
+/*
+ * Return the maximum number of lines in labels
+ */
+int xbaeCalculateLabelMaxLines(String *labels, XmString *xmlabels, int n_labels)
 {
-        char *nl;
-        if (label == NULL) {
-                label_line->lines = 0;
-                label_line->lengths = NULL;
-        } else {
-                /*
-                 * First count the number of lines in the label
-                 */
-                label_line->lines = 1;
-                nl = label;
-                while ((nl = strchr(nl, '\n')) != NULL) {
-                        nl++;
-                        label_line->lines++;
-                }
-
-                /*
-                 * Now malloc a lengths array of the correct size.
-                 */
-                label_line->lengths = (int *) XtMalloc(label_line->lines * sizeof(int));
-
-                /*
-                 * An entry in the lengths array is the length of that line (substring).
-                 */
-
-                /*
-                 * Handle the case of one line (no strchr() needed)
-                 */
-                if (label_line->lines == 1)
-                        label_line->lengths[0] = strlen(label);
-                else {
-                        int i;
-
-                        nl = label;
-                        i = 0;
-                        while ((nl = strchr(nl, '\n')) != NULL) {
-                                label_line->lengths[i] = nl - label;
-                                nl++;
-                                label = nl;
-                                i++;
-                        }
-                        label_line->lengths[i] = strlen(label);
-                }
-        }
-}
-
-int xbaeCalculateLabelMaxLines(ColumnLabelLines label_lines, int n_label_lines) {
         int i;
         int max_lines = 0;
-        if (label_lines) {
-                max_lines = label_lines[0].lines;
-                for(i = 1; i < n_label_lines; i++) {
-                        if (label_lines[i].lines > max_lines) {
-                                max_lines = label_lines[i].lines;
+        if (labels || xmlabels) {
+                for(i = 0; i < n_labels; i++) {
+                        int n_lines = 0;
+                        if (xmlabels && xmlabels[i]) {
+                                n_lines = XmStringLineCount(xmlabels[i]);
+                        } else if (labels && labels[i]) {
+                                n_lines = xbaeCountLines(labels[i]);
+                        }
+                        if (n_lines > max_lines) {
+                                max_lines = n_lines;
                         }
                 }
         }
@@ -714,16 +759,8 @@ Boolean xbaeEventToMatrixXY(XbaeMatrixWidget mw, XEvent * event, int *x, int *y)
         }
 
         if (event->xbutton.window == XtWindow(TextChild(mw))) {
-                Position tx, ty;
-                
-                XtVaGetValues(TextChild(mw), XmNx, &tx, XmNy, &ty, NULL);
-                *x += tx;
-                *y += ty;
-
-                if (mw->matrix.current_parent != (Widget) mw) {
-                        *x += mw->matrix.current_parent->core.x;
-                        *y += mw->matrix.current_parent->core.y;
-                }
+                *x += xbaeColumnToMatrixX(mw, mw->matrix.current_column);
+                *y += xbaeRowToMatrixY(mw, mw->matrix.current_row);
         } else if (event->xbutton.window == XtWindow(LeftClip(mw))) {
                 *x += FIXED_COLUMN_POSITION(mw);
                 *y += NON_FIXED_ROW_POSITION(mw);
@@ -804,8 +841,8 @@ Boolean xbaeMatrixXtoColumn(XbaeMatrixWidget mw, int *x, int *column)
                     && *x < FIXED_COLUMN_POSITION(mw) + VISIBLE_FIXED_COLUMN_WIDTH(mw)) {
                         *x -= FIXED_COLUMN_POSITION(mw);
                         /* Check if we are not in the horizontal fill */
-                        if(*x > COLUMN_POSITION(mw, mw->matrix.fixed_columns)) {
-                                *column = FILL_COLUMN(mw);
+                        if(*x >= COLUMN_POSITION(mw, mw->matrix.fixed_columns)) {
+                                *column = mw->matrix.fixed_columns - 1;
                                 *x -= COLUMN_POSITION(mw, *column);
                                 return True;
                         }
@@ -813,10 +850,10 @@ Boolean xbaeMatrixXtoColumn(XbaeMatrixWidget mw, int *x, int *column)
                            && *x < TRAILING_FIXED_COLUMN_POSITION(mw)
                                    + VISIBLE_TRAILING_FIXED_COLUMN_WIDTH(mw)) {
                         *x -= TRAILING_FIXED_COLUMN_POSITION(mw) 
-                              - COLUMN_POSITION(mw, TRAILING_HORIZ_ORIGIN(mw));
+                              - COLUMN_POSITION(mw, TRAILING_COLUMN_ORIGIN(mw));
                         /* Check if we are not in the horizontal fill */
-                        if(*x > COLUMN_POSITION(mw, mw->matrix.columns)) {
-                                *column = FILL_COLUMN(mw);
+                        if(*x >= COLUMN_POSITION(mw, mw->matrix.columns)) {
+                                *column = mw->matrix.columns - 1;
                                 *x -= COLUMN_POSITION(mw, *column);
                                 return True;
                         }
@@ -825,12 +862,12 @@ Boolean xbaeMatrixXtoColumn(XbaeMatrixWidget mw, int *x, int *column)
                         *x -= NON_FIXED_COLUMN_POSITION(mw) - HORIZ_ORIGIN(mw)
                              - FIXED_COLUMN_WIDTH(mw);
                         /* Check if we are not in the horizontal fill */
-                        if(*x > COLUMN_POSITION(mw, TRAILING_HORIZ_ORIGIN(mw))) {
-                                *column = FILL_COLUMN(mw);
+                        if(*x >= COLUMN_POSITION(mw, TRAILING_COLUMN_ORIGIN(mw))) {
+                                *column = TRAILING_COLUMN_ORIGIN(mw) - 1;
                                 *x -= COLUMN_POSITION(mw, *column);
                                 return True;
                         }
-                  } else {
+                } else {
                         *column = -2;
                         return False;
                 }
@@ -865,8 +902,8 @@ Boolean xbaeMatrixYtoRow(XbaeMatrixWidget mw, int *y, int *row)
                 if (*y >= FIXED_ROW_POSITION(mw) && *y < FIXED_ROW_POSITION(mw) + VISIBLE_FIXED_ROW_HEIGHT(mw)) {
                         *y -= FIXED_ROW_POSITION(mw);
                         /* Check if we are not in the vertical fill */
-                        if (*y > ROW_POSITION(mw, mw->matrix.fixed_rows)) {
-                                *row = FILL_ROW(mw);
+                        if (*y >= ROW_POSITION(mw, mw->matrix.fixed_rows)) {
+                                *row = mw->matrix.fixed_rows - 1;
                                 *y -= ROW_POSITION(mw, *row);
                                 return True;
                         }
@@ -874,10 +911,10 @@ Boolean xbaeMatrixYtoRow(XbaeMatrixWidget mw, int *y, int *row)
                            && *y < TRAILING_FIXED_ROW_POSITION(mw) 
                                    + VISIBLE_TRAILING_FIXED_ROW_HEIGHT(mw)) {
                         *y -= TRAILING_FIXED_ROW_POSITION(mw) 
-                              - ROW_POSITION(mw, TRAILING_VERT_ORIGIN(mw));
+                              - ROW_POSITION(mw, TRAILING_ROW_ORIGIN(mw));
                         /* Check if we are not in the vertical fill */
-                        if (*y > ROW_POSITION(mw, mw->matrix.rows)) {
-                                *row = FILL_ROW(mw);
+                        if (*y >= ROW_POSITION(mw, mw->matrix.rows)) {
+                                *row = mw->matrix.rows - 1;
                                 *y -= ROW_POSITION(mw, *row);
                                 return True;
                         }
@@ -885,8 +922,8 @@ Boolean xbaeMatrixYtoRow(XbaeMatrixWidget mw, int *y, int *row)
                            && *y < NON_FIXED_ROW_POSITION(mw) + VISIBLE_NON_FIXED_HEIGHT(mw)) {
                         *y -= NON_FIXED_ROW_POSITION(mw) - VERT_ORIGIN(mw) - FIXED_ROW_HEIGHT(mw);
                         /* Check if we are not in the vertical fill */
-                        if (*y > ROW_POSITION(mw, TRAILING_VERT_ORIGIN(mw))) {
-                                *row = FILL_ROW(mw);
+                        if (*y >= ROW_POSITION(mw, TRAILING_ROW_ORIGIN(mw))) {
+                                *row = TRAILING_ROW_ORIGIN(mw) - 1;
                                 *y -= ROW_POSITION(mw, *row);
                                 return True;
                         }
@@ -947,7 +984,7 @@ int xbaeColumnToMatrixX(XbaeMatrixWidget mw, int column)
         } else if (IS_LEADING_FIXED_COLUMN(mw, column)) {
                 x = FIXED_COLUMN_POSITION(mw) + COLUMN_POSITION(mw, column);
         } else if (IS_TRAILING_FIXED_COLUMN(mw, column)) {
-                x = TRAILING_FIXED_COLUMN_POSITION(mw) + COLUMN_POSITION(mw, column) - COLUMN_POSITION(mw, TRAILING_HORIZ_ORIGIN(mw));
+                x = TRAILING_FIXED_COLUMN_POSITION(mw) + COLUMN_POSITION(mw, column) - COLUMN_POSITION(mw, TRAILING_COLUMN_ORIGIN(mw));
         } else {
                 x = NON_FIXED_COLUMN_POSITION(mw) + COLUMN_POSITION(mw, column) - COLUMN_POSITION(mw, mw->matrix.fixed_columns) - HORIZ_ORIGIN(mw);
         }
@@ -964,7 +1001,7 @@ int xbaeRowToMatrixY(XbaeMatrixWidget mw, int row)
         } else if (IS_LEADING_FIXED_ROW(mw, row)) {
                 y = FIXED_ROW_POSITION(mw) + ROW_POSITION(mw, row);
         } else if (IS_TRAILING_FIXED_ROW(mw, row)) {
-                y = TRAILING_FIXED_ROW_POSITION(mw) + ROW_POSITION(mw, row) - ROW_POSITION(mw, TRAILING_VERT_ORIGIN(mw));
+                y = TRAILING_FIXED_ROW_POSITION(mw) + ROW_POSITION(mw, row) - ROW_POSITION(mw, TRAILING_ROW_ORIGIN(mw));
         } else {
                 y = NON_FIXED_ROW_POSITION(mw) + ROW_POSITION(mw, row) - ROW_POSITION(mw, mw->matrix.fixed_rows) - VERT_ORIGIN(mw);
         }
@@ -998,42 +1035,113 @@ Widget xbaeRowColToClipXY(XbaeMatrixWidget mw, int row, int column, int *x, int
         return w;
 }
 
-void xbaePositionCellWidget(XbaeMatrixWidget mw, int row, int column) {
+void xbaePositionWidgetOverCell(XbaeMatrixWidget mw, Widget w, int row, int column)
+{
+        Widget new_parent = xbaeGetCellClip(mw, row, column);
+
+        int x = xbaeColumnToMatrixX(mw, column) + mw->matrix.cell_shadow_thickness;
+        int y = xbaeRowToMatrixY(mw, row) + mw->matrix.cell_shadow_thickness;
+        int width = COLUMN_WIDTH(mw, column) - 2 * mw->matrix.cell_shadow_thickness;
+        int height = ROW_HEIGHT(mw, row) - 2 * mw->matrix.cell_shadow_thickness;
+
+        if (IS_FILL_COLUMN(mw, column) && mw->matrix.horz_fill) {
+                width += EMPTY_WIDTH(mw);
+        }
+
+        if (IS_FILL_ROW(mw, row) && mw->matrix.vert_fill) {
+                height += EMPTY_HEIGHT(mw);
+        }
+        
+        XtConfigureWidget(w,
+           /* position */ x, y,
+           /* size     */ width, height,
+           /* bw       */ XtBorderWidth(w));
+
+        if (XtWindow(new_parent)) {
+                if (new_parent != (Widget) mw) {
+                        /* The widget is drawn in one of the clip windows */
+                        x -= new_parent->core.x;
+                        y -= new_parent->core.y;
+                }
+                XReparentWindow(XtDisplay(mw), XtWindow(w), XtWindow(new_parent), x, y);
+        }
+}
+
+void xbaePositionCellWidget(XbaeMatrixWidget mw, int row, int column)
+{
         Widget cellWidget = mw->matrix.per_cell ? mw->matrix.per_cell[row][column].widget : NULL;
         
         if (cellWidget && XtIsRealized(cellWidget) && XtIsManaged(cellWidget)) {
-                Widget parent_window = xbaeGetCellClip(mw, row, column);
-                
-                int x = xbaeColumnToMatrixX(mw, column) + mw->matrix.cell_shadow_thickness;
-                int y = xbaeRowToMatrixY(mw, row) + mw->matrix.cell_shadow_thickness;
-                int width = COLUMN_WIDTH(mw, column) - 2 * mw->matrix.cell_shadow_thickness;
-                int height = ROW_HEIGHT(mw, row) - 2 * mw->matrix.cell_shadow_thickness;
-
-                XtConfigureWidget(cellWidget,
-                   /* position */ x, y,
-                   /* size     */ width, height,
-                   /* bw       */ XtBorderWidth(cellWidget));
-
-                if (parent_window != (Widget) mw) {
-                        /* The widget is drawn in one of the clip windows */
-                        int window_x = x - parent_window->core.x;
-                        int window_y = y - parent_window->core.y;
-                        
-                        XReparentWindow(XtDisplay(mw), XtWindow(cellWidget), XtWindow(parent_window), window_x, window_y);
-                }
+                xbaePositionWidgetOverCell(mw, cellWidget, row, column);
         }
 }
 
-void xbaeRefocusTextChild(XbaeMatrixWidget mw) {
-        if (xbaeIsCellVisible(mw, mw->matrix.current_row, mw->matrix.current_column)) {
-                Widget focus = XmGetFocusWidget((Widget) mw);
-                if (focus == NULL || focus == ClipChild(mw)) {
-                        XmProcessTraversal(TextChild(mw), XmTRAVERSE_CURRENT);
-                }
+void xbaePositionTextChild(XbaeMatrixWidget mw)
+{
+        xbaePositionWidgetOverCell(mw, TextChild(mw),
+                                   mw->matrix.current_row, 
+                                   mw->matrix.current_column);
+
+        /*
+         * The textChild is now visible
+         */
+        mw->matrix.text_child_is_mapped = True;
+}
+
+void xbaeHideCellWidget(XbaeMatrixWidget mw, Widget w)
+{
+        Dimension width = XtWidth(w);
+        Dimension height = XtHeight(w);
+        Dimension bw = XtBorderWidth(w);
+
+        XtConfigureWidget(w,
+                         /* position */	-1 - width - bw, -1 - height - bw,
+                         /* size */	width, height,
+                         /* bw */	bw);
+}
+
+void xbaeHideTextChild(XbaeMatrixWidget mw)
+{
+        /*
+         * Let Xt believe the textChild is still visible so we can traverse to it
+         */
+        XtConfigureWidget(TextChild(mw),
+           /* position */ 0, 0,
+           /* size     */ 1, 1,
+           /* bw       */ XtBorderWidth(TextChild(mw)));
+        
+        /*
+         * But have X not show the window
+         */
+        if (XtIsRealized(TextChild(mw)))
+		XReparentWindow(XtDisplay(mw), XtWindow(TextChild(mw)), XtWindow(mw), -1, -1);
+        
+        /*
+         * The textChild is now hidden
+         */
+        mw->matrix.text_child_is_mapped = False;
+}
+
+void xbaeSetInitialFocus(XbaeMatrixWidget mw)
+{
+        int row = xbaeTopRow(mw);
+        int column = xbaeLeftColumn(mw);
+        Widget widget;
+        
+        if (mw->matrix.per_cell && mw->matrix.per_cell[row][column].widget) {
+                widget = mw->matrix.per_cell[row][column].widget;
+        } else {
+                widget = TextChild(mw);
+        }
+
+        if (widget != mw->manager.initial_focus) {
+                XtVaSetValues((Widget) mw, 
+                              XmNinitialFocus, widget,
+                              NULL);
         }
 }
 
-void xbaeSaneRectangle(XbaeMatrixWidget mw, XRectangle *rect_p,int rs, int cs, int re, int ce){
+void xbaeSaneRectangle(XbaeMatrixWidget mw, XRectangle *rect_p, int rs, int cs, int re, int ce){
 
         int x1,x2,y1,y2;
 
@@ -1110,34 +1218,6 @@ void xbaeObjectUnlock(Widget w)
 #endif
 }
 
-/* The following 2 functions are here to provide information about the
-   version of this library - since we may often build it as a shared/dynamic
-   library it's important to have this builtin */
-const char *XbaeGetVersionTxt(void)
-{
-        return XbaeVersionTxt;
-}
-
-int XbaeGetVersionNum(void)
-{
-        return XbaeVersion;
-}
-
-/* The following 2 functions contain/return info about the
-   M*tif/LessTif version this library was built with.
-   Building against one toolkit and linking on runtime against
-   another is likely to fail! */
-const char *XbaeGetXmVersionTxt(void)
-{
-        return XmVERSION_STRING;
-}
-
-int XbaeGetXmVersionNum(void)
-{
-        return XmVersion;
-}
-
-
 /* Some string functions, since strcasecmp()/strncasecmp() are
    not always available */
 int _xbaeStrcasecmp(const char *s1, const char *s2)
@@ -1191,17 +1271,17 @@ int _xbaeStrncasecmp(const char *s1, const char *s2, size_t count)
 #endif
 }
 
-void xbaeScrollRows(XbaeMatrixWidget mw, Boolean Left, int step)
+void xbaeScrollRows(XbaeMatrixWidget mw, Boolean Up, int step)
 {
         int value, slider_size, increment, page_increment, limit;
-        XtVaGetValues(VertScrollChild(mw), (Left ? XmNminimum : XmNmaximum), &limit, NULL);
+        XtVaGetValues(VertScrollChild(mw), (Up ? XmNminimum : XmNmaximum), &limit, NULL);
         XmScrollBarGetValues(VertScrollChild(mw), &value, &slider_size, &increment,
                              &page_increment);
-        if (Left)
+        if (Up) {
                 XmScrollBarSetValues(VertScrollChild(mw),
                                      ((value - step < limit) ? limit : value - step), slider_size,
                                      increment, page_increment, True);
-        else {
+        } else {
                 limit -= slider_size;
                 XmScrollBarSetValues(VertScrollChild(mw),
                                      ((value + step > limit) ? limit : value + step), slider_size,
@@ -1209,55 +1289,20 @@ void xbaeScrollRows(XbaeMatrixWidget mw, Boolean Left, int step)
         }
 }
 
-void xbaeScrollColumns(XbaeMatrixWidget mw, Boolean Up, int step)
+void xbaeScrollColumns(XbaeMatrixWidget mw, Boolean Left, int step)
 {
         int value, slider_size, increment, page_increment, limit;
-        XtVaGetValues(HorizScrollChild(mw), (Up ? XmNminimum : XmNmaximum), &limit, NULL);
+        XtVaGetValues(HorizScrollChild(mw), (Left ? XmNminimum : XmNmaximum), &limit, NULL);
         XmScrollBarGetValues(HorizScrollChild(mw), &value, &slider_size, &increment,
                              &page_increment);
-        if (Up)
+        if (Left) {
                 XmScrollBarSetValues(HorizScrollChild(mw),
                                      ((value - step < limit) ? limit : value - step), slider_size,
                                      increment, page_increment, True);
-        else {
+        } else {
                 limit -= slider_size;
                 XmScrollBarSetValues(HorizScrollChild(mw),
                                      ((value + step > limit) ? limit : value + step), slider_size,
                                      increment, page_increment, True);
         }
 }
-
-
-/*
- * ARCAD SYSTEMHAUS
- *   june,29 2004
- */
-String xbaeGetCellStringValue(XbaeMatrixWidget mw, int row, int column)
-{
-        String string = "\0";
-        if (mw->matrix.per_cell) {
-                if (mw->matrix.per_cell[row][column].CellValue)
-                        string = mw->matrix.per_cell[row][column].CellValue;
-        }
-        return string;
-}
-
-Pixmap xbaeGetCellPixmap(XbaeMatrixWidget mw, int row, int column)
-{
-        Pixmap pixmap = XmUNSPECIFIED_PIXMAP;
-        if (mw->matrix.per_cell) {
-                if (mw->matrix.per_cell[row][column].pixmap != XmUNSPECIFIED_PIXMAP)
-                        pixmap = mw->matrix.per_cell[row][column].pixmap;
-        }
-        return pixmap;
-}
-
-Pixmap xbaeGetCellPixmapMask(XbaeMatrixWidget mw, int row, int column)
-{
-        Pixmap mask = XmUNSPECIFIED_PIXMAP;
-        if (mw->matrix.per_cell) {
-                if (mw->matrix.per_cell[row][column].mask != XmUNSPECIFIED_PIXMAP)
-                        mask = mw->matrix.per_cell[row][column].mask;
-        }
-        return mask;
-}
